#!/usr/bin/env python3

import sys
from fractions import Fraction
from typing import Optional, Tuple, Iterable, List


ZERO = Fraction(0)


class Point:
    def __init__(self, x: Fraction, y: Fraction) -> None:
        self.x = x
        self.y = y

    def __sub__(self, other: 'Point') -> 'Point':
        return Point(self.x - other.x, self.y - other.y)

    def __mul__(self, other: Fraction) -> 'Point':
        return Point(self.x * other, self.y * other)

    def __repr__(self) -> str:
        x = float(self.x)
        y = float(self.y)
        return f'({x}, {y})'


def dot(a: Point, b: Point) -> Fraction:
    return a.x * b.x + a.y * b.y


def cross(a: Point, b: Point, c: Optional[Point] = None) -> Fraction:
    if c is None:
        return a.x * b.y - a.y * b.x
    else:
        return cross(b - a, c - a)


def len2(a: Point) -> Fraction:
    return a.x * a.x + a.y * a.y


def dist2(a: Point, b: Point) -> Fraction:
    return len2(a - b)


class Halfspace:
    def __init__(self, v1: Point, v2: Point) -> None:
        assert v1 != v2
        self.verts = (v1, v2)

    def __call__(self, point: Point) -> Fraction:
        return cross(self.verts[0], self.verts[1], point)

    def __repr__(self) -> str:
        return f'{self.verts[0]!r} - {self.verts[1]!r}'


def intersect(a: Halfspace, b: Halfspace) -> Point:
    d0 = a(b.verts[0])
    d1 = a(b.verts[1])
    assert d0 != d1
    return (b.verts[1] * d0 - b.verts[0] * d1) * (1 / (d0 - d1))


class Polygon:
    def __init__(self, verts: Iterable[Point]) -> None:
        self.verts = list(verts)
        self.edges = [Halfspace(self.verts[i - 1], self.verts[i]) for i in range(len(self.verts))]
        self._check_invariants()

    def _check_invariants(self):
        assert len(self.verts) == len(self.edges)
        for i in range(len(self.verts)):
            a = self.verts[i - 1]
            b = self.verts[i]
            assert self.edges[i](a) == 0
            assert self.edges[i](b) == 0

    def clip(self, h: Halfspace) -> None:
        d = [h(v) for v in self.verts]
        if all(dist <= 0 for dist in d):
            self.verts = []
            self.edges = []
            return
        nverts: List[Point] = []
        nedges: List[Halfspace] = []
        for i in range(len(self.verts)):
            if d[i] > 0 and d[i - 1] <= 0:
                cut = intersect(h, self.edges[i])
                nedges.append(h)
                nedges.append(self.edges[i])
                nverts.append(cut)
                nverts.append(self.verts[i])
            elif d[i] <= 0 and d[i - 1] > 0:
                cut = intersect(h, self.edges[i])
                nedges.append(self.edges[i])
                nverts.append(cut)
            elif d[i] > 0 and d[i - 1] > 0:
                nedges.append(self.edges[i])
                nverts.append(self.verts[i])
        self.verts = nverts
        self.edges = nedges
        self._check_invariants()

    def __repr__(self) -> str:
        return repr(self.verts)


def main():
    if len(sys.argv) >= 2:
        sys.stdin = open(sys.argv[1], 'r')
    fields = sys.stdin.readline().split()
    dx = Fraction(fields[0])
    dy = Fraction(fields[1])
    n = int(fields[2])
    corners = [Point(ZERO, ZERO),
               Point(dx, ZERO),
               Point(dx, dy),
               Point(ZERO, dy)]
    poly = Polygon(corners)
    domes = []
    for i in range(n):
        x, y = [int(f) for f in sys.stdin.readline().split()]
        domes.append(Point(Fraction(x), Fraction(y)))
    order = [int(f) - 1 for f in sys.stdin.readline().split()]
    for i in range(n - 1):
        h = Halfspace(domes[order[i + 1]], domes[order[i]])
        poly.clip(h)
    for i in range(2, n):
        poly.clip(Halfspace(domes[order[i]], domes[order[0]]))
    area = Fraction(0)
    for i in range(len(poly.verts)):
        area += cross(poly.verts[i - 1], poly.verts[i])
    area /= 2
    if not area:
        print('0')
    else:
        print('{:.6f}'.format(abs(float(area))))


if __name__ == '__main__':
    main()
