#include <assert.h>
#include <inttypes.h>
#include <math.h>
#include <stdio.h>
#include <algorithm>
#include <vector>

// cf. https://web.stanford.edu/class/cs97si/09-computational-geometry.pdf

using namespace std;

typedef __int128 int128;
typedef long double Double;

struct Point { int x, y; };
struct FracPoint {
  Double x, y;
  Double dist0() const { return hypotl(x, y); }
};
struct Line {
  // Half-plane to "right" of p->q is a*x+b*y+c <= 0
  Line(const Point &p, const Point &q)
    : p(p), q(q),
      a(p.y - q.y),
      b(q.x - p.x),
      c((int128)p.x * (int128)q.y - (int128)p.y * (int128)q.x) {}
  bool is_on(const Point &s) const {
    return a * (int128)s.x + b * (int128)s.y + c == 0;
  }
  const Point p, q;
  const int128 a, b, c;
};
vector<Point> towers;
vector<int> order;
int n, dx, dy;

bool read_data() {
  if (scanf("%d %d %d", &dx, &dy, &n) != 3) return false;
  towers.resize(n);
  order.resize(n);
  for (Point &p : towers) {
    assert(scanf("%d %d", &p.x, &p.y) == 2);
  }
  for (int &k : order) {
    assert(scanf("%d", &k) == 1);
    assert(1 <= k && k <= n);
    --k;
  }
  return true;
}

inline int128 det2(int128 a, int128 b, int128 c, int128 d) {
  return a * d - b * c;
}

inline Double det2l(Double a, Double b, Double c, Double d) {
  return a * d - b * c;
}

void compute_intersections(const vector<Line> &lines,
                           vector<FracPoint> *intersections) {
  for (size_t i = 0; i < lines.size(); ++i) {
    const Line &li = lines[i];
    for (size_t j = i + 1; j < lines.size(); ++j) {
      const Line &lj = lines[j];

      // Intersect li and lj
      const int128 det = det2(li.a, li.b, lj.a, lj.b);
      if (det == 0) continue;  // no intersection (parallel)
      const int128 det_x = det2(-li.c, li.b, -lj.c, lj.b);
      const int128 det_y = det2(li.a, -li.c, lj.a, -lj.c);

      bool on_correct_sides = true;
      for (size_t k = 0; k < lines.size(); ++k) {
        if (k == i || k == j) continue;
        const Line &lk = lines[k];
        if ((det > 0 && (lk.a * det_x + lk.b * det_y + lk.c * det) > 0) ||
            (det < 0 && (lk.a * det_x + lk.b * det_y + lk.c * det) < 0)) {
          on_correct_sides = false;
          break;
        }
      }
      if (on_correct_sides) {
        intersections->push_back({
            (Double) det_x / (Double) det,
              (Double) det_y / (Double) det
              });
      }
    }
  }
}

Double ccw(const FracPoint &a, const FracPoint &b, const FracPoint &c) {
  return (b.x - a.x) * (c.y - a.y) - (b.y - a.y) * (c.x - a.x);
}

void compute_convex_hull(vector<FracPoint> *intersections,
                         vector<FracPoint> *hull) {
  sort(intersections->begin(), intersections->end(), [] (const FracPoint &a, const FracPoint &b) {
      if (a.x != b.x) return a.x < b.x;
      return a.y > b.y;
    });

  // Copy all points with minimal x-coordinate (in decreasing y-order)
  const Double min_x = intersections->front().x;
  if (intersections->back().x == min_x) {
    // Single point or vertical line
    *hull = *intersections;
    return;
  }
  int left_side = 0;
  while ((*intersections)[left_side].x == min_x) {
    hull->push_back((*intersections)[left_side++]);
  }

  // Sort remaining points by increasing y/x "angle"
  const Double min_y = hull->back().y;
  sort(intersections->begin() + left_side, intersections->end(),
       [min_x, min_y] (const FracPoint &a, const FracPoint &b) {
         const Double alpha = (a.y - min_y) / (a.x - min_x);
         const Double beta = (b.y - min_y) / (b.x - min_x);
         if (alpha != beta) return alpha < beta;
         return false;
       });

  hull->push_back((*intersections)[left_side]);
  for (int i = left_side + 1; i < (int)intersections->size(); ++i) {
    while (true) {
      assert(hull->size() >= 2);
      const FracPoint &a = (*hull)[hull->size() - 2];
      const FracPoint &b = hull->back();
      const FracPoint &c = (*intersections)[i];
      if (ccw(a, b, c) < 0.0) {
        hull->pop_back();
      } else {
        hull->push_back(c);
        break;
      }
    }
  }
}

void find_closest(const FracPoint &p, const FracPoint &q,
                  double *dist, double *x, double *y) {
  if (p.dist0() < q.dist0()) {
    *dist = p.dist0(); *x = p.x; *y = p.y;
  } else {
    *dist = q.dist0(); *x = q.x; *y = q.y;
  }

  const Double det = det2l(q.y - p.y, p.x - q.x, p.x - q.x, p.y - q.y);
  if (det == 0.0) return;  // single point
  const Double t = det2l(q.y - p.y, p.x, p.x - q.x, p.y) / det;
  if (0.0 <= t && t <= 1.0) {
    const FracPoint opt = { p.x + t * (q.x - p.x), p.y + t * (q.y - p.y) };
    assert(opt.dist0() <= *dist);
    *dist = opt.dist0();
    *x = opt.x;
    *y = opt.y;
  }
}

Double area(const vector<FracPoint> &v) {
  const int n = v.size();
  Double a = 0.0;
  for (int i = 0; i < n; ++i) {
    a += (v[(i+1)%n].y + v[i].y) * (v[(i+1)%n].x - v[i].x);
  }
  return fabsl(a / 2.0);
}

void process_data() {
  if (n == 1) {
    printf("%.6Lf\n", (Double)dx * (Double)dy);
    return;
  }

  // Half-planes that need to contain our vantage point
  vector<Line> lines;
  for (int i = 1; i < n; ++i) {
    lines.emplace_back(towers[order[i-1]], towers[order[i]]);
    if (i != 1) lines.emplace_back(towers[order[0]], towers[order[i]]);
  }
  // Add boundary half-planes as conditions
  const vector<Point> boundary = { { 0, 0 }, { 0, dy }, { dx, dy }, { dx, 0 } };
  for (int i = 0; i < 4; ++i) {
    lines.emplace_back(boundary[i], boundary[(i+1)%4]);
  }

  // Compute intersection points that are possible vantage points
  vector<FracPoint> intersections;
  compute_intersections(lines, &intersections);

  if (intersections.empty()) {
    printf("%.6Lf\n", 0.0L);
    return;
  }
  assert(intersections.size() >= 3);

  // Compute convex hull
  vector<FracPoint> hull;
  compute_convex_hull(&intersections, &hull);
  assert(hull.size() >= 3);

  const Double a = area(hull);
  printf("%.6Lf\n", a);
}

int main() {
  while (read_data()) process_data();
  return 0;
}
