#include <algorithm>
#include <functional>
#include <iostream>
#include <map>
#include <vector>
using namespace std;

// Vector-like structure that can compute a given operation on ranges in log(N) time.
// Op can be any associative operation with an identity.  Empty slots default to the identity.
template<typename T, typename OpType = T(*)(const T&, const T&)> struct SegmentTree {
  T ident;
  OpType op;
  vector<vector<T>> v;
  SegmentTree(vector<T>&& u, const T& ident = T(), OpType op = OpType())
      : ident(ident), op(op), v(1, move(u)) { resize(size()); fixFull(0, size()); }
  SegmentTree(initializer_list<T> il, const T& ident = T(), OpType op = OpType())
      : SegmentTree(vector<T>(il), ident, op) {}
  SegmentTree(const vector<T>& u, const T& ident = T(), OpType op = OpType())
      : SegmentTree(vector<T>(u), ident, op) {}
  explicit SegmentTree(size_t sz, const T& ident = T(), OpType op = OpType())
      : SegmentTree(vector<T>(sz, ident), ident, op) {}
  explicit SegmentTree(const T& ident = T(), OpType op = OpType())
      : SegmentTree({}, ident, op) {}

  // Technical stuff that gives some standard vector functionality.
  inline bool empty() const { return v[0].empty(); }
  inline size_t size() const { return v[0].size(); }
  inline void clear() { resize(0); }
  inline void pop_back() { resize(size()-1); }
  inline void push_back(const T& val) { v[0].push_back(val); resize(size()); fix(size()-1); }
  inline void push_back(T&& val) { v[0].push_back(move(val)); resize(size()); fix(size()-1); }
  template<typename... Args> void emplace_back(Args&&... args) {
    v[0].emplace_back(forward<Args>(args)...); resize(size()); fix(size()-1);
  }
  typedef const T& const_reference;
  // Use proxy so assignment works.  Warning: "auto x = v[3];" is not copied by value!
  struct reference {
    SegmentTree* st;
    size_t idx;
    inline reference operator=(const T& val) { st->v[0][idx] = val; st->fix(idx); return *this; }
    inline reference operator=(T&& val) { st->v[0][idx] = move(val); st->fix(idx); return *this; }
    inline operator const_reference() const { return st->v[0][idx]; }
  };
  inline reference operator[](size_t idx) { return {this, idx}; };
  inline reference front() { return {this, 0}; }
  inline reference back() { return {this, size()-1}; }
  inline const_reference operator[](size_t idx) const { return v[0][idx]; };
  inline const_reference front() const { return v[0].front(); }
  inline const_reference back() const { return v[0].back(); }
  // Only support const iterators, for simplicity.
  inline typename vector<T>::const_iterator begin() const { return v[0].cbegin(); }
  inline typename vector<T>::const_iterator end() const { return v[0].cend(); }

  void resize(size_t sz) {
    v[0].resize(sz, ident);
    for (size_t i = 1; sz /= 2, true; i++) {
      if (i >= v.size() || !sz) v.resize(i+1);
      if (sz == v[i].size()) break;
      if (sz > v[i].size()) v[i].push_back(op(v[i-1][v[i].size()*2], v[i-1][v[i].size()*2+1]));  // Slight hack.
      v[i].resize(sz, ident);
    }
  }
  inline void fix(size_t x) {  // Aggregate all parents of x.
    for (size_t i = 1; x /= 2, x < v[i].size(); i++) v[i][x] = op(v[i-1][2*x], v[i-1][2*x+1]);
  }
  inline void fixFull(size_t s, size_t e) {  // Aggregate all nodes touching [s..e).
    for (size_t i = 1; s /= 2, e = min((e+1)/2, v[i].size()), s < e; i++) {
      for (size_t x = s; x < e; x++) v[i][x] = op(v[i-1][2*x], v[i-1][2*x+1]);
    }
  }

  // Rewrite ranges, more efficiently than one element at a time.
  void SetRange(size_t s, size_t e, const T& elem) {
    for (size_t x = s; x < e; x++) v[0][x] = elem;
    fixFull(s, e);
  }
  template<typename InputIterator> void Copy(size_t s, InputIterator first, InputIterator last) {
    size_t e;
    for (e = s; first != last; e++, ++first) v[0][e] = *first;
    fixFull(s, e);
  }

  // Calc computes op[s..e) in logarithmic time.
  T Calc(size_t s, size_t e) {
    T lv = ident, rv = ident;
    for (size_t i = 0; s < e; i++, s = (s+1)/2, e /= 2) {
      if (s&1) lv = op(lv, v[i][s]);
      if (e&1) rv = op(v[i][e-1], rv);
    }
    return op(lv, rv);
  }
};

int main() {
  int N;
  while (cin >> N) {
    vector<vector<int>> v(N, vector<int>(4));
    vector<int> mx(3);
    for (int n = 0; n < N; n++) {
      for (int i = 0; i < 3; i++) {
        cin >> v[n][i];
        mx[i] = max(mx[i], v[n][i]);
      }
      v[n][3] = n;
    }

    // Single-coordinate cost.
    vector<int64_t> cost(N);
    for (int n = 0; n < N; n++) {
      for (int i = 0; i < 3; i++) cost[n] = max(cost[n], int64_t(mx[i] - v[n][i]));
    }

    // Double-coordinate cost.
    for (int xi = 0; xi < 3; xi++) {
      int yi = (xi+1)%3, zi = (xi+2)%3;
      vector<int> yc(N);
      for (int n = 0; n < N; n++) yc[n] = v[n][yi];
      sort(yc.begin(), yc.end());
      SegmentTree<int> yzsum(N, 0, [] (const int& a, const int& b) { return max(a, b); });
      map<int, int> zyn;
      sort(v.begin(), v.end(), [&] (const vector<int>& a, const vector<int>& b) { return a[xi] < b[xi]; });
      for (int n = 0; n < N; n++) {
        auto it = zyn.upper_bound(v[n][zi]);
        while (it != zyn.begin()) {
          --it;
          if (v[it->second][yi] > v[n][yi]) break;
          it = zyn.erase(it);
        }
        it = zyn.upper_bound(v[n][zi]);
        int ys = lower_bound(yc.begin(), yc.end(), v[n][yi]) - yc.begin();
        int ye = (it == zyn.end() ? 0 : upper_bound(yc.begin(), yc.end(), v[it->second][yi]) - yc.begin());
        cost[v[n][3]] = max(cost[v[n][3]], int64_t(yzsum.Calc(ys, ye) - v[n][yi] - v[n][zi]));
        if (it == zyn.end() || v[it->second][yi] < v[n][yi]) {
          yzsum[ys] = v[n][yi] + v[n][zi];
          zyn[v[n][zi]] = n;
        }
      }
    }

    // Triple-coordinate cost is unneeded; the dominating phone will always be better.

//for (auto x : cost) cout << x << ' ';
//cout << endl;
    int64_t ret = 1e18, retn = -1;
    for (int n = 0; n < N; n++) if (cost[n] < ret) { ret = cost[n]; retn = n; }
    cout << ret << ' ' << retn+1 << endl;
  }
}
