#!/usr/bin/env ruby

require 'set'

# Max input size
N = 200000

Nsqrt = Math.sqrt(N).to_i

# Executable for computing answers
BIN = 'submissions/accepted/matthias'

class Generator
  def initialize(file_path)
    if !File.directory?(file_path)
      STDERR.puts "Run from above \"#{file_path}\" directory"
      exit 1
    end
    @file_no = 1
    @file_path = file_path
    Dir["#{file_path}/*"].each do |f|
      File.delete(f)
    end

    # Compile solution if we have no binary yet
    if !File.exist?(BIN)
      `g++ -O3 -o #{BIN} #{BIN}.cc`
    end
  end

  # n random points in [1..10^9]^3 cube
  def random(n)
    points = n.times.map { [ rand(1..1000000000), rand(1..1000000000), rand(1..1000000000) ] }
    output(points)
  end

  # n random points on sphere with center (0,0,0)
  def random_sphere(n)
    points = Set.new
    while points.count < n do
      points << sphere_point(rand, rand)
    end
    output(points.to_a)
  end

  # All points on plane defined by (x+y+z = n)
  def plane111(n)
    points = []
    (1..(n-2)).each do |x|
      (1..(n-x-1)).each do |y|
        points << [x,y,n-x-y]
      end
    end
    output(points)
  end

  # Parts of xy-, xz-, yz-planes that have x+y+z <= n+2
  def sides(n)
    points = []
    (1..n).each do |i|
      (1..(n-i+1)).each do |j|
        points << [i,j,1]
        points << [i,1,j]
        points << [1,i,j]
      end
    end
    output(points.uniq.shuffle)
  end

  # n points on regular n-gon centered at (0,0) (i.e. roughly a
  # quarter of a 4n-gon)
  def regular_n_gon(n)
    points = n.times.map do |i|
      sphere_point(i.to_f / (n-1).to_f, 0.0)
    end
    output(points)
  end

  # n points on sphere centered at (0,0,0), computed by stepping
  # through longitude and latitude by constent angles
  def regular_n_sphere(n)
    points = []
    n.times do |i|
      n.times do |j|
        points << sphere_point(i.to_f / (n-1).to_f, j.to_f / (n-1).to_f)
      end
    end
    output(points.uniq)
  end

  # Check validity and output the points in the next input file; also
  # compute answer if executable is available
  def output(points)
    # Check input sizes, uniqueness
    raise "Bad number of points #{points.count}" if points.count > N || points.count < 2
    raise if points.uniq.count != points.count

    # File names
    in_file = "#{@file_path}/#{format('%02d', @file_no)}.in"
    ans_file = "#{@file_path}/#{format('%02d', @file_no)}.ans"
    @file_no += 1

    f = File.open(in_file, 'w')
    f << "#{points.count}\n"
    points.each do |x,y,z|
      # Check dimensions
      raise "#{x} #{y} #{z}" if x < 1 || x > 1000000000 || y < 1 || y > 1000000000 || z < 1 || z > 1000000000
      f << "#{x} #{y} #{z}\n"
    end
    f.close

    # Compute output if we have an executable
    if File.exist?(BIN)
      File.write(ans_file, `#{BIN} < #{in_file}`)
    end
  end

  private

  # Point on sphere centered at given angles (that range 0..1)
  def sphere_point(alpha, beta)
    pi_2 = Math::PI / 2.0
    c = Math.cos(pi_2 * beta)
    x = 999999999.0 * Math.cos(pi_2 * alpha) * c + 1.01
    y = 999999999.0 * Math.sin(pi_2 * alpha) * c + 1.01
    z = 999999999.0 * Math.sin(pi_2 * beta) + 1.01
    [ x.to_i, y.to_i, z.to_i ]
  end
end

srand(2020_06_25)

g = Generator.new('data/secret')

# Explicit examples
g.output([[1,1,1], [2,2,2]])
g.output([[2,1,2], [1,2,1]])
g.output([[1,11,1], [11,1,1], [1,1,1]])
g.output([[1,11,1], [11,1,1], [2,2,1]])
g.output([[1,11,1], [11,1,1], [6,6,1]])
g.output([[1,11,1], [11,1,1], [10,10,1]])

# All permutations of coordinates in a point set (i.e. symmetric in
# x/y/z dimensions)
g.output([100,200,200].permutation.uniq)
g.output([30,40,50].permutation)
g.output([1000000000,999999999,999999998].permutation)
g.output([1000000000,500000000,1].permutation)
g.output([30,40,50].permutation.to_a + [[39,39,39]])
g.output([30,40,50].permutation.to_a + [20,45,55].permutation.to_a + [10,50,60].permutation.to_a)
g.output([30,40,50].permutation.to_a + [35,40,44].permutation.to_a + [37,40,41].permutation.to_a)

# Lines from origin (pointing away and toward origin)
g.output(N.times.map { |i| [i+1,i+1,i+1] })
g.output(N.times.map { |i| [i+1,i+1,i+1] }.reverse)

# Plane - orthogonal to y-axis
g.output((Nsqrt*Nsqrt).times.map { |i| [(i%Nsqrt)+1, 10000, (i/Nsqrt)+1] }.shuffle)

# Plane - orthogonal to (1,1,1) vector
g.plane111(10)
g.plane111(100)
g.plane111(500)

# V-shape (center is optimal)
g.output(100000.times.map { |i| [ 1+i, 1000000000-i*9999, 12345 ] } +
         100000.times.map { |i| [ 1000000000-i*9999, 1+i, 12345 ] })

# 3 major axes
g.output((N/3).times.map { |i| [i+2,1,1] } + (N/3).times.map { |i| [1,i+2,1] } + (N/3).times.map { |i| [1,1,i+2] })

# 3 lines in directions (1,1,0), (1,0,1), (0,1,1)
g.output((N/3).times.map { |i| [i+2,i+2,1] } + (N/3).times.map { |i| [1,i+2,i+2] } + (N/3).times.map { |i| [i+2,1,i+2] })

# Parts of xy/yz/xz-planes
g.sides(10)
g.sides(100)
g.sides(250)

# 2D - regular n-gon
g.regular_n_gon(100)
g.regular_n_gon(10000)
g.regular_n_gon(N)

# 3D - sphere
g.regular_n_sphere(10)
g.regular_n_sphere(100)
g.regular_n_sphere(Nsqrt)

# Random subset of points on sphere
g.random_sphere(10)
g.random_sphere(100)
g.random_sphere(20000)

# Random points in [1,10^9] cube
g.random(100)
g.random(10000)
g.random(N)
