import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.regex.Pattern;

/**
 * ICPC World Finals 2019
 * Interaction format checker: QCQC
 * 
 * This tool validates the format of sample interactions included in the sample
 * data directory and automatically included into the problem statement.
 */
public class QcqcInteractionFormatChecker {
	static final Pattern PAT_IN_NUM = Pattern.compile("[<][1-9][0-9]*");
	static final Pattern PAT_OUT = Pattern.compile("[>][a-z].*");
	static final Pattern PAT_OUT_TEST = Pattern.compile("[>]test( ([1-9][0-9]*|0))+");
	static final Pattern PAT_OUT_ANSWER = Pattern.compile("[>]answer [01]+");
	static final Pattern PAT_IN_RESULT = Pattern.compile("[<][01-]+");
	
	static final int MAX_MACHINES = 11;
	static final int MAX_TESTS = 12;
	
	BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
	
	public static void main(String[] args) throws Exception {
		QcqcInteractionFormatChecker instance = new QcqcInteractionFormatChecker();
		instance.testFormat();
		if (instance.in.readLine() != null) throw new IllegalArgumentException("excess output");
		System.exit(42);
	}
	
	String readLine(Pattern pat) throws Exception {
		String s = in.readLine();
		if (s == null) throw new IllegalArgumentException("data missing, expecting another line: " + pat);
		if (!pat.matcher(s).matches()) throw new IllegalArgumentException("pattern " + pat + " does not match the line: " + s);
		return s;
	}
	
	void testFormat() throws Exception {
		String s = readLine(PAT_IN_NUM);
		Integer tests = Integer.parseInt(s.substring(1));
		for (int i = 0; i < tests; ++i)
			testInstance();
	}
	
	int n, round;
	int[][] tested;
	char[][] result;
	
	void testInstance() throws Exception {
		String s = readLine(PAT_IN_NUM);
		n = Integer.parseInt(s.substring(1));
		if (n < 1 || n > MAX_MACHINES) throw new IllegalArgumentException("too many machines");
		round = 0;
		tested = new int[MAX_TESTS][];
		result = new char[MAX_TESTS][];
		
		for (;;) {
			s = readLine(PAT_OUT);
			if (s.charAt(1) == 'a') break;
			testTest(s);
		}
		testAnswer(s);
	}
	
	void testTest(String s) throws Exception {
		if (round >= MAX_TESTS) throw new IllegalStateException("too many tests!");
		if (!PAT_OUT_TEST.matcher(s).matches()) throw new IllegalArgumentException("invalid test format: " + s);
		String[] ss = s.split(" ");
		if (ss.length != n+1) throw new IllegalArgumentException("invalid test length, should be " + n + ": " + s);
		tested[round] = new int[n]; Arrays.fill(tested[round], -1);
		boolean[] used = new boolean[n];
		for (int i = 0; i < n; ++i) {
			int t = Integer.parseInt(ss[i+1]);
			if (t < 0 || t > n) throw new IllegalArgumentException("machine number too high: " + s);
			if (t > 0) {
				if (used[t-1]) throw new IllegalArgumentException("machine " + t + " tested twice: " + s);
				used[t-1] = true;
			}
			tested[round][i] = t-1;
		}
		
		s = readLine(PAT_IN_RESULT);
		if (s.length() != n+1) throw new IllegalArgumentException("invalid result length, should be " + n + ": " + s);
		for (int i = 0; i < n; ++i)
			if ((s.charAt(i+1) == '-') != (tested[round][i] < 0)) throw new IllegalArgumentException("result NOPs do not match: " + s);
		result[round++] = s.substring(1).toCharArray();
	}

	void testAnswer(String s) {
		if (!PAT_OUT_ANSWER.matcher(s).matches()) throw new IllegalArgumentException("invalid answer format: " + s);
		s = s.split(" ")[1];
		if (s.length() != n) throw new IllegalArgumentException("invalid answer length, should be " + n + ": " + s);
		
		int maskans = Integer.parseInt(s, 2);
		if (!isAnswerValid(maskans)) throw new IllegalArgumentException("the answer is not valid: " + s);
		// we want the answer to be unique, otherwise it might be confusing
		for (int m = 0; m < (1<<n); ++m) if (m != maskans) {
			if (isAnswerValid(m)) throw new IllegalArgumentException("the answer is valid but not unique: " + s + " x " + Integer.toString(m, 2));
		}
	}
	
	boolean bit(int mask, int i) {
		return (mask & (1<<(n-1-i))) != 0;
	}
	
	boolean isAnswerValid(int mask) {
		if (Integer.bitCount(mask) <= n/2) return false;
		for (int j = 0; j < n; ++j)
			if (bit(mask, j))
				for (int i = 0; i < round; ++i)
					if (tested[i][j] >= 0)
						if (bit(mask, tested[i][j]) != (result[i][j] == '1'))
							return false;
		return true;
	}
}
