#!/usr/bin/env python3

import sys
import os
from enum import Enum
import random
from typing import List, Iterable, Tuple, Generator, TypeVar


MAXB = 500
MAXN = 100
T = TypeVar('T')

class Shuffle:
    NONE = 0
    REVERSE = 1
    FULL = 2


def do_shuffle(items: Iterable[T], shuffle: Shuffle) -> List[T]:
    if shuffle == Shuffle.NONE:
        return list(items)
    elif shuffle == Shuffle.REVERSE:
        return list(reversed(items))
    else:
        out = list(items)
        random.shuffle(out)
        return out


def malfunction(trg, rprob, gprob, bprob):
    if rprob == 0.0 or random.random() >= rprob:
        return trg
    elif trg == '1':
        return '1' if gprob == 1.0 or random.random() < gprob else '0'
    else:
        return '0' if bprob == 0.0 or random.random() >= bprob else '1'


def make_batch(n: int, good: int, add: int = 0, rem: int = 0,
               rprob: float = 0.0, gprob: float = 1.0, bprob: float = 0.0,
               block_shuffle: Shuffle = Shuffle.NONE,
               shuffle: Shuffle = Shuffle.FULL,
               diagonal1: bool = True) -> Tuple[str, List[str]]:
    assert good * 2 > n
    assert add + good <= n
    assert rem <= good
    bad = n - good
    state = ['1'] * good + ['0'] * bad
    alt = ['1'] * (good - rem) + ['0'] * rem + ['1'] * add + ['0'] * (bad - add)
    perms = [list(range(0, good - rem)),
             list(range(good - rem, good)),
             list(range(good, good + add)),
             list(range(good + add, n))]
    perms = [do_shuffle(p, block_shuffle) for p in perms]
    perm = [item for p in perms for item in p]
    perm = do_shuffle(perm, shuffle)
    state = [state[x] for x in perm]
    alt = [alt[x] for x in perm]
    answers = [None] * n
    for i in range(n):
        if state[i] == '1':
            answers[i] = list(state)
        else:
            answers[i] = [malfunction(a, rprob, gprob, bprob) for a in alt]
    if diagonal1:
        for i in range(n):
            answers[i][i] = '1'

    return ''.join(state), [''.join(a) for a in answers]


def generate_batches() -> Generator[Tuple[str, List[str]], None, None]:
    # Exercise range of problem sizes and good counts
    for n in range(1, MAXN):
        for good in range(n // 2 + 1, n):
            yield make_batch(n, good)
            yield make_batch(n, good, rprob=0.5, gprob=0.7, bprob=0.3)
            yield make_batch(n, good, rprob=1.0, gprob=0.0, bprob=1.0)

    # Two groups, each internally claiming to be good
    for n in range(MAXN - 1, MAXN + 1):
        for good in range(n // 2 + 1, n // 2 + 3):
            for repeat in range(200):
                yield make_batch(n, good, rprob=1.0, gprob=0.0, bprob=1.0)

    # Malfunctioning machines always tell the truth
    for n in range(MAXN - 1, MAXN + 1):
        for good in range(n // 2 + 1, n // 2 + 3):
            for repeat in range(200):
                yield make_batch(n, good)

    # Malfunctioning machines always tell the truth, except about a few
    # machines where it tries to confuse.
    for n in range(MAXN - 1, MAXN + 1):
        for good in range(n // 2 + 1, n // 2 + 5):
            for add in range(0, 5):
                for rem in range(0, 5):
                    for repeat in range(100):
                        yield make_batch(n, good, add=add, rem=rem)

    # Non-random reordering
    for add in range(0, 3):
        for rem in range(0, 3):
            yield make_batch(MAXN, MAXN // 2 + 1 + rem, add=add, rem=rem,
                             block_shuffle=Shuffle.FULL, shuffle=Shuffle.NONE)
            yield make_batch(MAXN, MAXN // 2 + 1 + rem, add=add, rem=rem,
                             block_shuffle=Shuffle.FULL, shuffle=Shuffle.REVERSE)
            yield make_batch(MAXN, MAXN // 2 + 1 + rem, add=add, rem=rem,
                             block_shuffle=Shuffle.NONE, shuffle=Shuffle.REVERSE)
            yield make_batch(MAXN, MAXN // 2 + 1 + rem, add=add, rem=rem,
                             block_shuffle=Shuffle.REVERSE, shuffle=Shuffle.REVERSE)

    # All random parameters (except shuffles)
    for i in range(500):
        n = random.randint(1, MAXN)
        good = random.randint(n // 2 + 1, n)
        add = random.randint(0, n - good)
        rem = random.randint(0, good)
        rprob = random.random()
        gprob = random.random()
        bprob = random.random()
        diagonal1 = random.random() < 0.99
        yield make_batch(n, good, add=add, rem=rem, rprob=rprob, gprob=gprob, bprob=bprob,
                         diagonal1=diagonal1)


def generate_cases() -> Generator[List[Tuple[str, List[str]]], None, None]:
    cur = []
    for batch in generate_batches():
        cur.append(batch)
        if len(cur) == MAXB:
            yield cur
            cur = []
    if len(cur):
        yield cur


def main():
    if not os.path.exists('data/secret'):
        print('Run from top level directory (containing data/secret)', file=sys.stderr)
        sys.exit(1)
    random.seed(42565367854624)
    num = 1
    for case in generate_cases():
        with open('data/secret/{:03}_generated.in'.format(num), 'w') as f:
            print(len(case), file=f)
            for state, answers in case:
                print(len(state), state, file=f)
                for answer in answers:
                    print(answer, file=f)
        with open('data/secret/{:03}_generated.ans'.format(num), 'w') as f:
            pass
        num += 1


if __name__ == '__main__':
    main()
