// DFS in a tree; at each vertex v of the DFS tree, remember the shortest
// paths of four types:
//
//    - min1: shortest path visiting every node in the subtree rooted at v,
//      with v as an endpoint
//    - min2: shortest path visiting every node in the subtree with v as
//      an internal point of the path
//    - min3: shortest pair of paths visiting every node, with v as an endpoint
//      of at least one of the two paths
//    - min4 shortest pair of paths visiting every node in v's subtree
//
// Also remember loop = 2*(total weight of edges in v's subtree), the cost of
// a path entering and leaving v and visiting all vertices in v's subtree
//
// When backing out of v, compute these paths (or rather, their lengths)
// using various combinations of the paths in v's subtrees

import java.util.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.StringTokenizer;

class Edge {
  public int from,to;
  public int dist;
  public Edge(int f, int t, int d) {
    from = f; to = t; dist = d;
  }
}

class Vertex {
  public int id,parent;
  public int loop,min1,min2,min3,min4;
  public ArrayList<Edge> nbr;
  public Vertex(int i) {
    id = i;
    nbr = new ArrayList<Edge>();
  }
}

public class RSRSnow {
  public static BufferedReader in;
  public static StringTokenizer st;
  public static int n;
  public static Vertex[] gr;
  public static int min;

  public static void main(String[] args) throws IOException {
    in = new BufferedReader(new InputStreamReader(System.in));
    n = Integer.parseInt(in.readLine());

    gr = new Vertex[n];
    for (int i = 0; i < n-1; i++) {
      st = new StringTokenizer(in.readLine());
      int v = Integer.parseInt(st.nextToken())-1;
      int w = Integer.parseInt(st.nextToken())-1;
      int d = Integer.parseInt(st.nextToken());
      if (gr[v]==null) {
        gr[v] = new Vertex(v);
      }
      if (gr[w]==null) {
        gr[w] = new Vertex(w);
      }
      gr[v].nbr.add(new Edge(v,w,d));
      gr[w].nbr.add(new Edge(w,v,d));
    }

    min = Integer.MAX_VALUE;

    gr[0].parent = -1;
    dfs(0);

    System.out.println (Math.min(Math.min(gr[0].min2,gr[0].min3),gr[0].min4));
  }


  public static void dfs(int v) {

    int numch = gr[v].nbr.size()-1;
    if (gr[v].parent < 0) numch++;

    // Is this a leaf? If so, all values are zero:
    if (numch == 0) {
      gr[v].loop = gr[v].min1 = gr[v].min2 = gr[v].min3 = gr[v].min4 = 0;

      return;
    }

    int min1 = Integer.MAX_VALUE;
    int min2 = Integer.MAX_VALUE;
    int min3 = Integer.MAX_VALUE;
    int min4 = Integer.MAX_VALUE;
    int loop = 0;

    int sumloop = 0; // 2*sum of all subtree weights
    int sumdist = 0; // sum of all edge weights except parent

// The "gain" associated with a particular child of v is the amount we
// save by using a path of a particular type in that child when creating
// the path lengths for v. We keep the four best gains of each type (since
// we may need to combine paths of different types from different children):
//
// I didn't need all of these, but copy and paste made it easier to just
// duplicate code for min1!

    int min1gain[] = new int[]{0,0,0,0};
    int min1x[] = new int[]{-2,-2,-2,-2};
    Edge min1e[] = new Edge[]{null,null,null,null};

    int min2gain[] = new int[]{0,0,0,0};
    int min2x[] = new int[]{-2,-2,-2,-2};
    Edge min2e[] = new Edge[]{null,null,null,null};

// Splicing is using the edge from v to its child TWICE, effectively
// splicing v into the min2 path of its child:
    int splice2gain[] = new int[]{0,0,0,0};

    int splice2x[] = new int[]{-2,-2,-2,-2};
    Edge splice2e[] = new Edge[]{null,null,null,null};

    int min3gain[] = new int[]{0,0,0,0};
    int min3x[] = new int[]{-2,-2,-2,-2};
    Edge min3e[] = new Edge[]{null,null,null,null};

    int min4gain[] = new int[]{0,0,0,0};
    int min4x[] = new int[]{-2,-2,-2,-2};
    Edge min4e[] = new Edge[]{null,null,null,null};


///////
/// First pass: gather info about max gain, loop sums:
///////

    for (Edge e:gr[v].nbr) {
      int w = e.to;
      if (w == gr[v].parent) continue;
      gr[w].parent = v;
      sumdist += e.dist;
      dfs(w);
      sumloop += gr[w].loop;

      // For each child of v, see how much we can improve minimum by replacing
      // a loop through v by a path of a given type (min1, min2, etc.)

      int gain = gr[w].loop - gr[w].min1 + e.dist; // best new branch?
      insert(min1gain,min1x,min1e,gain,w,e);

      gain = gr[w].loop - gr[w].min2 + e.dist;

      insert (min2gain,min2x,min2e,gain,w,e);

      gain = gr[w].loop - gr[w].min2;
      insert(splice2gain,splice2x,splice2e,gain,w,e);

      gain = gr[w].loop - gr[w].min3 + e.dist;
      insert(min3gain,min3x,min3e,gain,w,e);

      gain = gr[w].loop - gr[w].min4;
      insert(min4gain,min4x,min4e,gain,w,e);
    }

    int all = sumloop + 2*sumdist;

////////
/// (1) Shortest path ENDING at v (i.e., one endpoint at v, other in
///     some subtree of v). Use edge with biggest min1 gain
////////

    min1 = all - min1gain[0];

////////
/// (2) Update shortest path THRU v (i.e., v internal to path) through w.
///     Uses "gain" info just collected.
////////

   ///////
   ///   case 1: "splice" into the min2 path of a child
   ///////

    min2 = all - splice2gain[0];

   ///////
   ///   case 2: use two min1 paths meeting at v
   ///////


    if (numch >= 2) {
      min2 = Math.min(min2,all - min1gain[0] - min1gain[1]);
    }

/////////
/// (3) Update min3 values (many cases):
/////////

   ///////
   ///   case 1: three min1 paths meet at v:
   ///////

    if (numch >= 3) { // both paths go thru v

      min3 = Math.min(min3,all - min1gain[0] - min1gain[1] - min1gain[2]);
    }

   ///////
   ///    case 2:  Connect to middle of a min2 path (creating a new min3 path):
   ///////

      min3 = Math.min(min3,all - min2gain[0]); // just one path goes thru v

   ///////
   ///    case 3: Connect to a min3 path (extend an existing min3 path):
   ///////

      min3 = Math.min(min3,all - min3gain[0]); // just one path goes thru v

   ///////
   ///    case 4: Combine a min1 path and "splice into" a min2 path:
   ///////

    if (numch >= 2) {

      // find best combination of a path ending at v and a spliced path thru v
      if (min1x[0] != splice2x[0]) { // best min1 tree != best splice tree
      min3 = Math.min(min3,all - min1gain[0] - splice2gain[0]);
      } else { //both bests in same tree, so try alternatives:
      min3 = Math.min(min3,all - min1gain[0] - splice2gain[1]);
      min3 = Math.min(min3,all - min1gain[1] - splice2gain[0]);
      }
    }

/////////
/// (4) Update min4 values (many cases):
/////////

   ///////
   ///    case 1: "splice into" a min4 path:
   ///////
      min4 = Math.min(min4,all - min4gain[0]);

   ///////
   ///    case 2: four min1 paths meet at v
   ///////
      if (numch >= 4) { // find four longest paths out of v
        min4 = Math.min(min4,all - min1gain[0] -min1gain[1] - min1gain[2] - min1gain[3]);
      }

   ///////
   ///    case 3: one min1 path and a min3 formed by adding an edge to the
   ///        middle of a min2
   ///////

      if (numch >= 2) {
        if (min1x[0] != min2x[0]) { // best min1 and best min2 in different trees
          min4 = Math.min(min4,all - min1gain[0] - min2gain[0]);
        } else { // try other possible combinations:
          min4 = Math.min(min4,all-min1gain[0]-min2gain[1]);
          min4 = Math.min(min4,all-min1gain[1]-min2gain[0]);
        }

   ///////
   ///    case 4: "splice" into two min2 paths
   ///////
        min4 = Math.min(min4, all - splice2gain[0] - splice2gain[1]);
  
   ///////
   ///    case 5: use one min1 path and extend an existing min3 path:
   ///////
        if (min1x[0] != min3x[0]) { // best min1 and min3 in different trees
        min4 = Math.min(min4,all-min1gain[0]-min3gain[0]);
      } else { // try all combinations
        min4 = Math.min(min4,all-min1gain[0]-min3gain[1]);
        min4 = Math.min(min4,all-min1gain[1]-min3gain[0]);
      }
    }
  
   ///////
   ///    case 6: two min1 paths and splice into a min2
   ///////

    if (numch >= 3) { // two paths and splice into a min2:
      for (int i = 0; i < 3; i++) {
        for (int j = 0; j < 3; j++) {
          for (int k = 0; k < 3; k++) {
            if (i==j) continue;
            if (min1e[i].to == splice2e[k].to || min1e[j].to == splice2e[k].to) continue;
            min4 = Math.min(min4,all-min1gain[i]-min1gain[j] - splice2gain[k]);
          }
        }
      }
    }

    min3 = Math.min(min3,min2);
    min4 = Math.min(min4,min3);

    gr[v].min1 = min1;
    gr[v].min2 = min2;
    gr[v].min3 = min3;
    gr[v].min4 = min4;
    gr[v].loop = all;
  }

  // Sort of borrowed from bmerry, but essentially just insertion sort
  // with a max of four values (smaller values get dropped after first four)
  public static void insert(int[] m, int[] x, Edge[] e, int g, int v, Edge f) {
      for (int i = 0; i < 4; i++) {
        if (g >= m[i]) {
          for (int j = 3; j > i; j--) {
            m[j] = m[j-1]; x[j] = x[j-1]; e[j] = e[j-1];
          }
          m[i] = g; x[i] = v; e[i] = f;
          return;
        }
      }
  }
}
