
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.StringTokenizer;
import java.util.function.Function;
import java.util.stream.IntStream;
import java.util.stream.Stream;

/**
 * ICPC World Finals 2020
 * Problem Candidate: Space Walls
 * 
 * This solution uses two types of 3D transformations to avoid code duplicity:
 * 
 * First, for each robot, the coordinates are "shuffled" in such a way that the
 * robot keeps its Z coordinate constant all the time, and goes in a counter-clockwise loop.
 * 
 * Second, for each move (straight line segment along one edge), the whole scene is rotated
 * such that the robot is apparently on the same side and in the same direction.
 * 
 * Each such move determines its final position (either a wall "up" or "down" or closing the loop)
 * and all intersections on that segment are remembered. Their coordinates are transformed
 * back (by a reversed transformation) to absolute coordinates.
 * 
 * Finally, for each crossing point, the earliest collision is found.
 */
public class SpaceWrong1 {
	static final boolean DEBUG = false;
	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	boolean hasNextToken() {
		while (!st.hasMoreTokens()) try {
			String line = input.readLine();
			if (line == null) return false;
			st = new StringTokenizer(line);
		} catch (Exception ex) { throw new RuntimeException(ex); }
		return true;
	}
	String nextToken() {
		return (!hasNextToken()) ? null : st.nextToken();
	}
	int nextInt() {
		return Integer.parseInt(nextToken());
	}
	public static void main(String[] args) throws Exception {
		new SpaceWrong1().run();
	}
	static void makeSure(boolean expr) { if (!expr) throw new IllegalStateException("precondition failed"); }
	static final int INF = 100000000;
	
	static class Point3 {
		final int x, y, z;
		Point3(int x, int y, int z) { this.x = x; this.y = y; this.z = z; }
		@Override public int hashCode() { return ((x * 31) + y) * 31 + z; }
		@Override public boolean equals(Object obj) {
			if (this == obj) return true;
			if (obj == null) return false;
			if (getClass() != obj.getClass()) return false;
			Point3 other = (Point3) obj;
			return (x == other.x) && (y == other.y) && (z == other.z);
		}
		@Override public String toString() { return "["+x+","+y+","+z+"]"; }
		Point3 cross(Point3 b) { return new Point3(this.y * b.z - this.z * b.y, this.z * b.x - this.x * b.z, this.x * b.y - this.y * b.x); }
	}
	static Point3 dir(String s) {
		int a = s.charAt(1) == '+' ? +1 : -1, c = s.charAt(0);
		return new Point3(c=='x'?a:0, c=='y'?a:0, c=='z'?a:0);
	}
	static final Point3 XP = dir("x+"), XM = dir("x-"), YP = dir("y+"), YM = dir("y-"), ZP = dir("z+"), ZM = dir("z-");
	static class Cube {
		final Point3 p1, p2;
		Cube(Point3 p1, Point3 p2) { this.p1 = p1; this.p2 = p2; }
	}
	
	static class CrossPoint {
		final Point3 cube, face;
		int mod, tim;
		CrossPoint(Point3 cube, Point3 face, int tim) { this.cube = cube; this.face = face; this.tim = tim; }
		@Override public int hashCode() { return cube.hashCode() * 31 + face.hashCode(); }
		@Override public boolean equals(Object obj) { return (obj instanceof CrossPoint)
				&& ((CrossPoint)obj).cube.equals(this.cube)
				&& ((CrossPoint)obj).face.equals(this.face);
		}
		@Override public String toString() { return "" + cube + face + tim+'/'+mod; }
	}
	Map<CrossPoint,List<CrossPoint>> crossings = new HashMap<>();
	long oppositeMin = Long.MAX_VALUE;

	interface Transform { Point3 tr(Point3 p); Point3 rev(Point3 p); }
	
	/* Transformation that makes a robot run in the same z coordinate and counter-clockwise. */
	static class Transform3 implements Transform {
		final int i0, i1, i2;
		Transform3(int i0, int i1, int i2) { this.i0 = i0; this.i1 = i1; this.i2 = i2; }
		public Point3 tr(Point3 p) { int[] pp = new int[]{p.x, p.y, p.z}; return new Point3(pp[i0], pp[i1], pp[i2]); }
		public Point3 rev(Point3 p) { int[] pp = new int[3]; pp[i0] = p.x; pp[i1] = p.y; pp[i2] = p.z; return new Point3(pp[0], pp[1], pp[2]); }
		@Override public String toString() { return "" + "xyz".charAt(i0) + "xyz".charAt(i1) + "xyz".charAt(i2); }
	}
	
	/* Rotating transformation for purposes of one robot path. */
	static class Transform2 implements Transform {
		final int xx, xy, yx, yy;
		Transform2(int xx, int xy, int yx, int yy) { this.xx = xx; this.xy = xy; this.yx = yx; this.yy = yy; }
		public Point3 tr(Point3 p) { return new Point3(p.x * xx + p.y * xy, p.x * yx + p.y * yy, p.z); }
		public Point3 rev(Point3 p) { return tr(tr(tr(p))); }
		Transform2 left() { return new Transform2(-yx, -yy, xx, xy); }
		Transform2 right() { return new Transform2(yx, yy, -xx, -xy); }
		@Override public boolean equals(Object obj) { return (obj instanceof Transform2) && ((Transform2)obj).tr(XP).equals(this.tr(XP)); }
	}
	
	Transform normTrans;
	Transform normalize(Point3 cdir) {
		return normTrans = cdir.z>0 ? new Transform3(0,1,2)
				: cdir.z<0 ? new Transform3(1,0,2)
				: cdir.y>0 ? new Transform3(2,0,1)
				: cdir.y<0 ? new Transform3(0,2,1)
				: cdir.x>0 ? new Transform3(1,2,0)
				: new Transform3(2,1,0);
	}
	Point3 ntr(Point3 p) { return normTrans.tr(p); }
	Point3 nrev(Point3 p) { return normTrans.rev(p); }
	
	Point3[] cub1, cub2;  // input cubes; cub2 is the last unit cube (input minus 1)
	Point3[] stp, stf, std; // input robots
	Cube[] byXmin, byYmin;
	int[] c1x, c2x, c1y, c2y;
	
	void run() {
		gcd(24,40);
		int n = nextInt(), k = nextInt();
		cub1 = new Point3[n]; cub2 = new Point3[n];
		for (int i = 0; i < n; ++i) {
			cub1[i] = new Point3(nextInt(), nextInt(), nextInt());
			cub2[i] = new Point3(nextInt()-1, nextInt()-1, nextInt()-1);
		}
		stp = new Point3[k]; stf = new Point3[k]; std = new Point3[k];
		for (int i = 0; i < k; ++i) {
			stp[i] = new Point3(nextInt(), nextInt(), nextInt());
			stf[i] = dir(nextToken()); std[i] = dir(nextToken());
		}
		for (int i = 0; i < k; ++i)
			oneRobot(i);
		Optional<Long> min = crossings.values().stream().map(this::findFirst)
				.filter(Objects::nonNull).reduce(Long::min);
		if (oppositeMin < Long.MAX_VALUE)
			min = Optional.of(Math.min(oppositeMin, min.orElse(Long.MAX_VALUE)));
		System.out.println(min.map(Function.<Object>identity()).orElse("ok"));
	}
	
	boolean inside(int a, int b, int v) { return Math.min(a, b) <= v && v <= Math.max(a, b); }
	
	void oneRobot(int r) {
		// transform the whole scene (coordinates shuffle) to work counter-clockwise inside the X-Y plane
		normalize(stf[r].cross(std[r]));
		Point3 p = ntr(stp[r]), f = ntr(stf[r]), d = ntr(std[r]);
		int pz = p.z;
		makeSure(f.cross(d).equals(ZP));
		if (DEBUG) System.out.println("#"+r + ":  " + nrev(p) + stf[r] + " " + normTrans + "-> " + p + f);
		
		// transform coordinates of other robots and remember all orthogonal planes
		int[] oxs = IntStream.range(0, stp.length)
				.filter(i -> ntr(stf[i]).x == 0 && ntr(std[i]).x == 0)
				.map(i -> ntr(stp[i]).x)
				.distinct().toArray();
		int[] oys = IntStream.range(0, stp.length)
				.filter(i -> ntr(stf[i]).y == 0 && ntr(std[i]).y == 0)
				.map(i -> ntr(stp[i]).y)
				.distinct().toArray();
		List<CrossPoint> crpts = new ArrayList<>();
		
		// detect robots in the opposite direction along the same path
		int[] opi = IntStream.range(0, stp.length)
				.filter(i -> ntr(stf[i]).cross(ntr(std[i])).equals(ZM))
				.filter(i -> ntr(stp[i]).z == pz)
				.toArray();
		Point3[] opp = Arrays.stream(opi).mapToObj(i -> ntr(stp[i])).toArray(Point3[]::new);
		Point3[] opf = Arrays.stream(opi).mapToObj(i -> ntr(stf[i])).toArray(Point3[]::new);
		
		// transform the whole shape and filter only those cuboids that intersect our Z coordinate
		int[] idxs = IntStream.range(0, cub1.length).filter(i -> inside(ntr(cub1[i]).z, ntr(cub2[i]).z, pz)).toArray();
		Point3[] c1 = Arrays.stream(idxs).mapToObj(i -> ntr(cub1[i])).toArray(Point3[]::new),
				c2 = Arrays.stream(idxs).mapToObj(i -> ntr(cub2[i])).toArray(Point3[]::new);
		
		// rotate the X-Y plane to always keep being on a "top" (y+) face aiming "left" (x-)
		Transform2 nxtrot = new Transform2(f.y,-f.x,f.x,f.y), rot0 = nxtrot;
		Point3 p0 = rot0.tr(p);
		makeSure(rot0.tr(f).equals(YP));
		boolean end = false;
		int len = 0;
		while (!end) {
			final Transform2 rot = nxtrot;
			if (DEBUG) System.out.println(len + "  " + nrev(p) + nrev(rot.rev(YP)));
			Point3 rp = rot.tr(p);
			int nxtup = IntStream.range(0, c1.length)
					.filter(i -> inside(rot.tr(c1[i]).y, rot.tr(c2[i]).y, rp.y + 1))  // only those that intersect the top
					.map(i -> Math.max(rot.tr(c1[i]).x, rot.tr(c2[i]).x) + 1)  // take their right end + 1 (we need to stop BEFORE the wall)
					.filter(v -> v <= rp.x)  // eliminate everything to the right
					.max().orElse(-INF);
			int nxtdn = IntStream.range(0, c1.length)
					.filter(i -> rp.y == Math.max(rot.tr(c1[i]).y, rot.tr(c2[i]).y))  // only those with the top at our position
					.map(i -> Math.min(rot.tr(c1[i]).x, rot.tr(c2[i]).x))  // take their left side
					.filter(v -> v <= rp.x)  // eliminate everything to the right
					.max().getAsInt();
			Point3 nxtrp;
			if (nxtup >= nxtdn) {
				nxtrp = new Point3(nxtup-1, rp.y+1, rp.z);
				nxtrot = rot.left();
			} else {
				nxtrp = new Point3(nxtdn, rp.y, rp.z);
				nxtrot = rot.right();
			}
			int dist = rp.x - Math.max(nxtup, nxtdn);
			if (len > 0 && rot0.equals(rot) && rp.y == p0.y && inside(rp.x - dist, rp.x, p0.x)) {
				dist = rp.x - p0.x - 1;  // loop ends, stop one step before the initial position
				if (dist < 0) break;
				end = true;
			}
			
			// find all crossings along the next path segment
			final Point3 pf = p;
			final int distf = dist, lenf = len;
			Stream<Point3> ots = (rot.rev(YP).x == 0) ? Arrays.stream(oxs).mapToObj(x -> new Point3(x, pf.y, pf.z))
					: Arrays.stream(oys).mapToObj(y -> new Point3(pf.x, y, pf.z));
			ots.map(rot::tr).map(q -> q.x)
					.filter(x -> inside(rp.x, rp.x - distf, x))
					.forEach(x -> crpts.add(regCrossing(rot.rev(new Point3(x, rp.y, rp.z)), rot.rev(YP), lenf + rp.x - x)));
			
			// find starting points of robots in the opposite direction
			IntStream.range(0, opi.length)
					.filter(i -> rot.tr(opf[i]).equals(YP))
					.map(i -> rot.tr(opp[i]).x)
					.filter(x -> inside(rp.x, rp.x - distf, x))
					.forEach(x -> regOpposite(lenf + rp.x - x));
			
			// real move to the next edge (or the end of loop)
			len += dist+1;
			p = rot.rev(nxtrp);
		}
		if (DEBUG) System.out.println(len);
		for (CrossPoint crs : crpts) crs.mod = len;
	}
	
	CrossPoint regCrossing(Point3 p, Point3 f, int tim) {
		CrossPoint crs = new CrossPoint(nrev(p), nrev(f), tim);
		if (DEBUG) System.out.println("reg: " + crs);
		List<CrossPoint> lst = crossings.get(crs);
		if (lst == null)
			crossings.put(crs, lst = new ArrayList<>());
		lst.add(crs);
		return crs;
	}

	void regOpposite(int tim) {
		oppositeMin = Math.min(oppositeMin, tim/2);
	}

	Long findFirst(List<CrossPoint> crpts) {
		if (DEBUG) System.out.print(crpts);
		if (crpts.size() > 100) return 4242L;
		Long res = null;
		for (CrossPoint crp1 : crpts) for (CrossPoint crp2 : crpts) if (crp1 != crp2) {
			Long t = findTime(crp1.tim, crp1.mod, crp2.tim, crp2.mod);
			if (t == null) continue;
			if (res == null || ((Long)res).longValue() > t.longValue())
				res = t;
		}
		if (DEBUG) System.out.println(" -> " + res);
		return res;
	}
	
	Long findTime(int t1, int m1, int t2, int m2) {
		if (t1 > t2) return findTime(t2, m2, t1, m1);
		int g = gcd(m1, m2);
		if ((t2-t1) % g != 0) return null;
		long k1 = (long) ga * (t2-t1) / g;
		long t = k1 * m1;
		return Long.valueOf(t1 + makemod(t, (long)m1*m2/g));
	}
	long makemod(long v, long m) {
		if (v >= 0) return v % m;
		return (v + (-v / m + 1) * m) % m;
	}
	
	int ga, gb;
	int gcd(int a, int b) {
		if (a == 0) {
			ga = 0; gb = 1; return b;
		}
		int g = gcd(b % a, a);
		int ga2 = ga;
		ga = gb - (b/a) * ga;
	    gb = ga2;
		return g;
	}
}
