
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * ICPC World Finals 2020
 * Sample Solution: Speed Limit Signs
 * <p>
 * Combine results of subtrees into a tree result.
 * The "result" is described by the {@link SubResult} class - see below.
 * <p>
 * This probably should be "the slowest solution we would like to accept".
 * Its time complexity is O(n^2) but it uses recursive DFS in Java
 * and plenty of objects allocated on the heap.
 * 
 * @author Martin Kacer
 */
public class SpeedLimitFasterMK {
	StringTokenizer st = new StringTokenizer("");
	BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
	boolean hasNextToken() {
		while (!st.hasMoreTokens()) try {
			String line = input.readLine();
			if (line == null) return false;
			st = new StringTokenizer(line);
		} catch (Exception ex) { throw new RuntimeException(ex); }
		return true;
	}
	String nextToken() {
		return (!hasNextToken()) ? null : st.nextToken();
	}
	int nextInt() {
		return Integer.parseInt(nextToken());
	}
	public static void main(String[] args) throws Exception {
		new SpeedLimitFasterMK().run();
	}
	
	static class Node {
		final int idx;
		final ArrayList<Edge> edges = new ArrayList<>();
		public Node(int idx) { this.idx = idx; }
		@Override public String toString() { return "N#"+(idx+1); }
	}
	static class Edge {
		final Node node;
		final int speed;
		Edge(Node node, int speed) { this.node = node; this.speed = speed; }
		@Override public String toString() { return "E->"+node+"("+speed+")"; }
	}
	Node[] graph;
	int n, csign;
	
	void run() {
		n = nextInt(); csign = nextInt();
		graph = new Node[n];
		for (int i = 0; i < n; ++i) graph[i] = new Node(i);
		for (int i = 1; i < n; ++i) {
			int u = nextInt()-1, v = nextInt()-1, s = nextInt();
			graph[u].edges.add(new Edge(graph[v], s));
			graph[v].edges.add(new Edge(graph[u], s));
		}
		System.out.println(solve(graph[0], null, null).bestCost());
	}

	/** The recursive step -- combine results of subtrees. */
	SubResult solve(Node cur, Edge inc, Node parent) {
		ArrayList<SubResult> subs = new ArrayList<>(cur.edges.size());
		int max = 0; long sum = 0;
		for (Edge e : cur.edges) {
			max = Math.max(max, e.speed);
			if (e.node == parent) continue;
			SubResult r = solve(e.node, e, cur);
			subs.add(r);
			sum += r.bestCost();
		}
		SubResult myres = new SubResult(cur.edges.size(), inc, sum);
		// gradually increase the speed and find next segments of the resulting function
		for (int s = max; s < Integer.MAX_VALUE; ) {
			long sc = 0; int sm = 0;
			for (SubResult sr : subs) {
				sc += sr.costFor(s) + (s - sr.inc.speed);
				sm += sr.marginalCost() + 1; // plus 1 for the respective edge itself
			}
			myres.addSegment(s, sc, sm);
			s = subs.stream().mapToInt(it -> it.nextSpeed()).reduce(Integer.MAX_VALUE, Integer::min);
		}
		//System.err.println(cur + " -> " + myres);
		return myres;
	}

	
	/**
	 * Description of a subtree result, which is basically a function
	 * mapping a speed limit of an incoming ("up") edge to the best possible solution
	 * of the whole subtree.
	 * Because of the nature of the problem, the result function is piecewise linear,
	 * consisting of left-closed right-open intervals.
	 */
	class SubResult {
		final Edge inc;
		final int deg;
		final long unbal;
		final ArrayList<Integer> si = new ArrayList<>(), mi = new ArrayList<>();
		final ArrayList<Long> ci = new ArrayList<>();
		int idx = 0;
		
		/**
		 * @param deg real degree of the current node, including the parent edge, if any.
		 * @param edge edge to the parent node. Null if this is the tree root.
		 * @param sum the sum of best solutions for all child nodes together.
		 */
		SubResult(int deg, Edge edge, long sum) { this.deg = deg; this.inc = edge; this.unbal = sum + deg * csign; }
		
		/** Basic cost for a given speed.
		 * @param speed speed of an "incoming edge" (the one from the node's parent).
		 *   Must not be less than the current linear segment - which means the speed of the parent
		 *   link at the beginning, and then the speed argument passed to this method for the last time.
		 * @return the best cost achievable with the given incoming edge speed, counting the whole subtree.
		 *   The cost may represent both balanced or unbalanced node, whichever is cheaper.
		 *   If the node is unbalanced, the cost will include the installation of all speed-limit
		 *   signs, including the bottom end of the incoming (parent) edge (but only if there exists any!).
		 *   However, note that the cost does NOT include any cost of upgrading the incoming edge.
		 *   Even when the result represents a balanced node, the incoming edge is upgrade is NOT counted.
		 */
		long costFor(int speed) {
			while (idx < si.size() && speed >= si.get(idx)) ++idx;
			if (idx == 0) return unbal; // incoming edge should have lower speed than our minimum
			return ci.get(idx-1) + (speed - si.get(idx-1)) * mi.get(idx-1);
		}
		
		/** Slope of the current linear segment.
		 * @return the "marginal cost" of increasing the incoming edge speed by 1 more km/h. 
		 *   The value corresponds to the last value returned by {@link #costFor}.
		 *   As for {@link #costFor}, the marginal cost does NOT include the incoming edge upgrade.
		 */
		int marginalCost() {
			return (idx == 0) ? 0 : mi.get(idx-1);
		}
		
		/** The left end of the next interval.
		 * @return the next "interesting" point of the result function greater than the last speed
		 *   passed to {@link #costFor}. MAX_VALUE if we are already in the last segment.
		 */
		int nextSpeed() {
			return (idx < si.size()) ? si.get(idx) : Integer.MAX_VALUE;
		}
		
		/** The best cost achievable for the whole subtree, without any limitation on the incoming edge.
		 * Unlike {@link #costFor}, this result will include the cost of upgrading the incoming edge,
		 * if it is necessary to do so.
		 */
		long bestCost() {
			long best = unbal;
			for (int i = 0; i < ci.size(); ++i)
				best = Math.min(best, ci.get(i) + (inc == null ? 0 : si.get(i) - inc.speed));
			return best;
		}
		
		/** Add a new linear segment to an already-existing function.
		 * This may only be invoked before the other (reading) methods. */
		void addSegment(int speed, long cost, int marg) {
			if (si.size() > 0 && si.get(si.size()-1) >= speed) {
				si.remove(si.size()-1); ci.remove(ci.size()-1); mi.remove(mi.size()-1);
			}
			if (marg == 0) { doAdd(speed, cost, 0); return; }
			long sd = (unbal - cost + marg-1) / marg;
			if (sd <= 0) doAdd(speed, unbal, 0);
			else {
				doAdd(speed, cost, marg);
				doAdd(speed + (int) sd, unbal, 0);
			}
		}
		private void doAdd(int speed, long cost, int marg) {
			if (!ci.isEmpty() && ci.get(ci.size()-1).longValue() == cost && mi.get(mi.size()-1).intValue() == marg) return;
			si.add(speed); ci.add(cost); mi.add(marg);
		}
		@Override public String toString() { return "SubResult/unbal="+unbal+si+ci+mi+"=>"+bestCost(); }
	}
}
