
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.function.IntToLongFunction;
import java.util.function.IntUnaryOperator;

/**
 * ICPC World Finals 2020: Speed Limit
 * Test data generator
 * 
 * @author Martin Kacer
 */
public class SpeedLimitGenerator {
	File DIR_TARGET = new File("data/secret/");
	PrintWriter data;
	int counter = 100;

	final Random RAND = new Random(2963915401764L);
	final int MAX_SIZE = 20000, MAX_COST = 100000;
	
	void open(String name) throws IOException {
		if (data != null) data.close();
		data = new PrintWriter(new File(DIR_TARGET, String.format("%03d", counter++) + "-" + name + ".in"));
	}
	void close() throws IOException { data.close(); data = null; }
	void open(String name, int n, int c) throws IOException {
		open(name);
		data.println(n + " " + c);
	}
	
	public static void main(String[] args) throws Exception {
		SpeedLimitGenerator inst = new SpeedLimitGenerator();
		if (args.length > 0) inst.DIR_TARGET = new File(args[0]);
		inst.generateAll();
		if (inst.data != null) inst.close();
	}
	
	void generate(String name, int n, int c, IntUnaryOperator parent, IntUnaryOperator speed, boolean shuffle) throws IOException {
		open("name", n, c);
		List<GenEdge> edges = new ArrayList<>(n);
		for (int i = 2; i <= n; ++i)
			edges.add(new GenEdge(i, parent.applyAsInt(i), speed.applyAsInt(i)));
		printShuffled(edges, shuffle, shuffle);
	}
	
	void genPath(String name, int n, int c, IntUnaryOperator speed) throws IOException {
		open(name, n, c);
		for (int i = 1; i < n; ++i)
			data.println(i + " " + (i+1) + " " + speed.applyAsInt(i));
	}
	
	void genPath(int n, int c, int smin, int smax) throws IOException {
		genPath("path-"+n, n, c, i -> smin + RAND.nextInt(smax-smin+1));
	}
	
	void printShuffled(List<GenEdge> edges, boolean shuffleEdges, boolean shuffleNodes) throws IOException {
		int n = edges.size() + 1;
		if (shuffleEdges)
			Collections.shuffle(edges, RAND);
		if (!shuffleNodes) {
			edges.forEach(data::println);
			return;
		}
		ArrayList<Integer> nums = new ArrayList<>(n);
		for (int i = 0; i < n; ++i) nums.add(i+1);
		Collections.shuffle(nums, RAND);
		for (GenEdge e : edges) {
			int u = e.u, v = e.v;
			if (RAND.nextBoolean()) {
				u = e.v; v = e.u;
			}
			data.println(nums.get(u-1) + " " + nums.get(v-1) + " " + e.s);
		}
	}
	
	void genBinary(int n, int c, IntUnaryOperator speed, boolean shuffle) throws IOException {
		open("binary-" + n, n, c);
		List<GenEdge> edges = new ArrayList<>(n);
		for (int i = 2; i <= n; ++i)
			edges.add(new GenEdge(i, i/2, speed.applyAsInt(i)));
		printShuffled(edges, shuffle, shuffle);
	}

	/** @param scale for index 1..n-1 return (relative) probability of using that node as a parent */
	void genRandom(int n, int c, IntToLongFunction scale, int smin, int smax, boolean shuffle) throws IOException {
		open("random-"+n, n, c);
		long[] scsum = new long[n]; int scpar[] = new int[n], sccnt = 0;
		scpar[sccnt] = 1; scsum[sccnt++] = scale.applyAsLong(1);
		List<GenEdge> edges = new ArrayList<>(n);
		for (int i = 2; i <= n; ++i) {
			long rpar = Math.abs(RAND.nextLong()) % scsum[sccnt-1];
			int ipar = Arrays.binarySearch(scsum, 0, sccnt, rpar+1);
			if (ipar < 0) ipar = -(ipar+1);
			edges.add(new GenEdge(i, scpar[ipar], smin + RAND.nextInt(smax-smin+1)));
			scsum[sccnt] = scsum[sccnt-1] + scale.applyAsLong(i);
			if (scsum[sccnt] > scsum[sccnt-1]) scpar[sccnt++] = i;
		}
		printShuffled(edges, shuffle, shuffle);
	}

	void generateAll() throws Exception {
		counter = 101;
		genPath(MAX_SIZE, 100, 100, 300);
		genPath("path-decreasing", MAX_SIZE, 1000, new IntUnaryOperator() {
			int speed = MAX_COST;
			@Override
			public int applyAsInt(int idx) {
				return speed -= 1 + RAND.nextInt(MAX_COST/MAX_SIZE);
			}
		});
		genPath("path-increasing", MAX_SIZE, 1000, new IntUnaryOperator() {
			int speed = 1;
			@Override
			public int applyAsInt(int idx) {
				return speed += 1 + RAND.nextInt(MAX_COST/MAX_SIZE);
			}
		});
		
		open("max-answer", MAX_SIZE, MAX_COST);
		for (int i = 4; i <= MAX_SIZE; ++i) {
			data.println("1 " + i + " 1");
		}
		data.println("1 2 2");
		data.println("1 3 " + MAX_COST);
		
		counter = 121;
		genBinary(MAX_SIZE, RAND.nextInt(100) + 100, i -> 50 + RAND.nextInt(100), true);
		
		counter = 201;
		genRandom(30, 100, i -> 1, 50, 200, true);
		genRandom(1000, 100, i -> 1, 50, 200, true);
		genRandom(1000, 100, i -> 1001-i, 50, 200, true);
		genRandom(MAX_SIZE, 5000, i -> MAX_SIZE/2-i/2, 2000, 12000, true);
		genRandom(MAX_SIZE, 10, i -> MAX_SIZE-i/2, 1, 20, true);
		genRandom(MAX_SIZE, 1000, i -> i, 300, 2500, true);
	}

	
	static class GenEdge {
		final int u, v, s;
		GenEdge(int u, int v, int s) { this.u = u; this.v = v; this.s = s; }
		@Override public String toString() { return u + " " + v + " " + s; }
	}
}
