import random
import sys
import math
import hashlib
random.seed(2020)

EXTREME_T_VALUE = 1e-5
DECIMAL_POINTS_PROB = 6

NRANGES = {
  'MIN': (1, 1),
  'SMALL': (1, 10),
  'MEDIUM': (10, 100),
  'LARGE': (100, 300),
  'HUGE': (300, 500),
  'MAX': (500, 500),
}

QRANGES = {
  'ZERO': (0, 0),
  'MIN': (1, 1),
  'SMALL': (1, 5),
  'HUGE': (300, 500),
  'MAX': (500, 500),
}

SRANGES = {
  'MIN': (1, 1),
  'SMALL': (2, 10),
  'ANY': (1, 500),
  'HUGE': (300, 500),
  'MAX': (500, 500),
}

def probrange(f, t):
  return random.uniform(f, t)

PROBS = {
  'MAX': lambda: 0.1,
  'LARGE': lambda: probrange(0.09, 0.1),
  'UNIFORM': lambda: probrange(0.0, 0.1),
  'MIXED': lambda: 0.1 if probrange(0, 1) > 0.5 else 0,
  'SMALL': lambda: probrange(0, 0.001),
  'ZERO': lambda: 0,
  'ONECOL1e-3': None,
  'ONECOL1': None,
  'ONECOL50': None,
  'ONEROW1e-3': None,
  'ONEROW1': None,
  'ONEROW50': None,
}

# How many standard deviations away from the mean are we. Note that the CLT
# approximation says that 4 stddevs away we should have roughly 1e-4 probability
# density, so I expect that within 4 stddevs we're safely within the limits.

# To really test behavior when we insert extreme values of T, we just calculate
# the probabilities (the MIN / MAX settings), and take the lowest / highest T
# that results in a probability over the limit.
TVAL = {
    'MAXVAL': (0, 0),
    'BITOFF': (-1, 1),
    'WIDE': (-4, 4),
    'MAX': None,
    'MIN': None,
}

def addMine(probs, p, cval, cutoff):
  # The formula is P'[k] = p * P[k-1] + (1-p) * P[k]
  probs.append(0.)
  for k in xrange(len(probs) - 1, cval, -1):
    probs[k] = p * probs[k-1] + (1. - p) * probs[k]
  probs[cval] *= (1. - p)
  while probs[-1] <= cutoff:
    probs.pop()
  while probs[cval] <= cutoff:
    cval += 1
  return cval

def generateExtremes(Ps, Qs):
  # Note: The point is to guarantee that the value we choose is no smaller
  # than 1e-9, because that's what we guarantee in the problem statement.
  # The "cutting off of small values" strategy is legit, because it gives a
  # lower bound on probabilities (since I only decrease values to zero), so
  # if we think something is larger than X, it really is.
  probs = [1.]
  cval = 0
  for pp in Ps:
    for qq in Qs:
      prob = pp + qq
      cval = addMine(probs, prob, cval, 1e-13)
  first = None
  second = None
  for x in xrange(cval, len(probs)):
    if probs[x] > 1.01 * EXTREME_T_VALUE:
      second = x
      if first is None:
        first = x
  return (first, second)

def cut(v, a, b):
  if v < a:
    return a
  if v > b:
    return b
  return v

def generateTest(nrange, qrange, srange, prob, tval):
  """Return value is a dict:
    - T: value of T
    - P: values of Pi. len(P) = M
    - Q: values of Qi. len(Q) = N
    - V: list of lists of pairs of ints - the queries.
          len(V) = Q, len(V[i]) = S.
  """
  result = {}
  ONECOL = ['ONECOL1e-3', 'ONECOL1', 'ONECOL50']
  ONEROW = ['ONEROW1e-3', 'ONEROW1', 'ONEROW50']
  N = random.randint(NRANGES[nrange][0], NRANGES[nrange][1])
  M = random.randint(NRANGES[nrange][0], NRANGES[nrange][1])
  Q = random.randint(QRANGES[qrange][0], QRANGES[qrange][1])
  # Generate the Ps and Qs
  result['P'] = []
  result['Q'] = []
  chosencol = -1
  chosenrow = -1
  if prob in ONECOL or prob in ONEROW:
    if prob == 'ONECOL1e-3' or prob == 'ONEROW1e-3':
      total = 2 * N * M * (10 ** (-DECIMAL_POINTS_PROB))
    elif prob == 'ONECOL1' or prob == 'ONEROW1':
      total = 1.
    else:
      total = 50.
    if prob in ONECOL:
      chosencol = random.randint(1, M)
      for i in xrange(M):
        result['P'].append(
          probrange(0.099, 0.1) if i == chosencol else probrange(0., total / (N*M)))
      for _ in xrange(N):
        result['Q'].append(probrange(0., total / (N * M)))
    else:
      chosenrow = random.randint(1, N)
      for _ in xrange(M):
        result['P'].append(probrange(0., total / (N * M)))
      for i in xrange(N):
        result['Q'].append(
          probrange(0.099, 0.1) if i == chosenrow else probrange(0., total / (N*M)))
  else:
    for _ in xrange(M):
      result['P'].append(PROBS[prob]())
    for _ in xrange(N):
      result['Q'].append(PROBS[prob]())
  # Generate T
  extremes = generateExtremes(result['P'], result['Q'])
  if tval in ['MIN', 'MAX']:
    result['T'] = extremes[0] if tval == 'MIN' else extremes[1]
  else:
    expval = 0.
    var = 0.
    for p in result['P']:
      for q in result['Q']:
        pr = p + q
        expval += pr
        var += pr - pr * pr
    std = math.sqrt(var)
    minval = cut(int(expval + TVAL[tval][0] * std), extremes[0], extremes[1])
    maxval = cut(int(expval + TVAL[tval][1] * std), extremes[0], extremes[1])
    result['T'] = random.randint(minval, maxval)
  # Generate the queries.
  result['V'] = []
  all_sq = []
  chos_sq = []
  for x in xrange(M):
    for y in xrange(N):
      if x == chosencol or y == chosenrow:
        chos_sq.append((x + 1, y + 1))
      else:
        all_sq.append((x + 1, y + 1))
  for _ in xrange(Q):
    S = cut(random.randint(SRANGES[srange][0], SRANGES[srange][1]), 0, M*N)
    v = []
    tak_chos = 0
    tak_all = 0
    for i in xrange(S):
      if tak_chos < len(chos_sq) and probrange(0, 1) < 0.99:
        choice = random.randint(tak_chos, len(chos_sq) - 1)
        chos_sq[tak_chos], chos_sq[choice] = chos_sq[choice], chos_sq[tak_chos]
        v.append((chos_sq[tak_chos][0], chos_sq[tak_chos][1]))
        tak_chos += 1
      else:
        choice = random.randint(tak_all, len(all_sq) - 1)
        all_sq[tak_all], all_sq[choice] = all_sq[choice], all_sq[tak_all]
        v.append((all_sq[tak_all][0], all_sq[tak_all][1]))
        tak_all += 1
    result['V'].append(v)
  return result

def testName(i, nrange, qrange, srange, prob, tval):
  return "{:03d}_NM_{}_Q_{}_S_{}_P_{}_T_{}".format(
      i, nrange, qrange, srange, prob, tval)

def printTest(name, nrange, qrange, srange, prob, tval):
  # Use the hash of the test name, except for the index, as the random seed.
  # This will cause repeated runs of the generator to generate the exact same
  # test for the same parameters (even if the indexing shifts), which is good
  # for debugging while writing this.
  float_format = "%.{}f".format(DECIMAL_POINTS_PROB)
  name_wo_index = "_".join(name.split("_")[1:])
  random.seed(int(hashlib.md5(name_wo_index).hexdigest(), 16))
  t = generateTest(nrange, qrange, srange, prob, tval)
  f = open("secret/" + name + ".in", "w")
  f.write("{} {} {} {}\n".format(len(t['P']), len(t['Q']), t['T'], len(t['V'])))
  f.write(" ".join([float_format % x for x in t['P']]) + "\n")
  f.write(" ".join([float_format % x for x in t['Q']]) + "\n")
  for a in t['V']:
    f.write("{} ".format(len(a)) + (
      " ".join([" ".join([str(y) for y in x]) for x in a])) + "\n")
  f.close()

def log(x):
  return math.log(x, 2)

def sqrt(x):
  return math.sqrt(x)

solutions = {
  'dk': lambda Q, Z, S: Q * Z * log(Z),
  'onufry_dp': lambda Q, Z, S: Q * Z * Z,
  'onufry_tree': lambda Q, Z, S: Q * Z * Z,
  'onufry_fourier': lambda Q, Z, S: Q * Z * log(Z) * log(Z),
  'onufry_reverse_fourier': lambda Q, Z, S: max(
      Q * Z * log(Z), Z * log(Z) * log(Z), Q * S * log(S) * log(S)),
  'onufry_cut_fourier': lambda Q, Z, S: max(
      Q * sqrt(Z) * log(Z), Z * log(Z) * log(Z), Q * S * log(S) * log(S)),
  'onufry_query_tree': lambda Q, Z, S: Q * S * sqrt(Z),
  'bmerry-faster': lambda Q, Z, S: Q * Z,
  'bmerry-slow': lambda Q, Z, S: Q * Z * Z,
  'dk_fourier_doubles': lambda Q, Z, S: Q * Z * log(Z),
  # That's cheaeting - there's an extra sqrt(Z) in there. But for now we trust
  # this solution, and we don't fully trust anything else, so...
  'onufry_cut_dp': lambda Q, Z, S: Q * Z,
  'onufry_cut_dp_aggressive': lambda Q, Z, S: Q * Z * sqrt(Z) / 3,
}
TLE = 'time_limit_exceeded'
WA = 'wrong_answer'
ACC = 'accepted'

# This also determines the order of the solutions, from the more trustworthy
# to the less trustworthy.
SOL_LOCATION = [
  ('onufry_query_tree', ACC),
  ('dk', WA, 'cc'),
  ('onufry_cut_dp_aggressive', TLE),
  ('bmerry-faster', ACC),
  ('onufry_cut_fourier', WA),
  ('onufry_dp', TLE),
  ('onufry_cut_dp', TLE),
  ('onufry_reverse_fourier', WA),
  ('dk_fourier_doubles', WA, 'cc'),
  ('onufry_fourier', WA),
  ('bmerry-slow', TLE),
  ('onufry_tree', TLE),
]

CUTOFF_FOR_TEST = 10000000000 # 10^10, which should be ~2-5 minutes.

def full_path(sol):
  if len(sol) == 2:
    extension = ".cpp"
  else:
    extension = "." + sol[2]
  return "../submissions/{}/{}{}".format(sol[1], sol[0], extension)

def cost(solution, test):
  Q = QRANGES[test[1]][1]
  Z = NRANGES[test[0]][1] * NRANGES[test[0]][1]
  S = SRANGES[test[2]][1]
  return solutions[solution](Q, Z, S)

def output_sample(solution, test):
  return (
    ["echo \" Using {} as key for test {}. \"".format(solution, test),
     "./{}.exe < ./secret/{}.in > ./secret/{}.ans || exit 1".format(
        solution, test, test),
     
    ],
    "./secret/{}.ans".format(test))

def valid_solutions(test):
  return [s[0] for s in SOL_LOCATION if cost(s[0], test) <= CUTOFF_FOR_TEST]

def output_line(solution, test, sample):
  return ["echo \"Running {} on {}\"".format(solution, test),
          "(time timeout 1m ./{}.exe < ./secret/{}.in > ./result || (echo \"Timed out\"; false)) && python compare.py {} result".format(
            solution, test, sample),
          "rm result"]

""" Arg order: N, Q, S, P, T """
tests = [
  ('MIN', 'MIN', 'MIN', 'MAX', 'MAXVAL'),
  ('MIN', 'MIN', 'MIN', 'ZERO', 'MAXVAL'),
  ('SMALL', 'ZERO', 'ANY', 'LARGE', 'MAXVAL'),
  ('SMALL', 'MIN', 'SMALL', 'LARGE', 'MAXVAL'),
  ('SMALL', 'MIN', 'SMALL', 'UNIFORM', 'WIDE'),
  ('SMALL', 'SMALL', 'SMALL', 'MIXED', 'BITOFF'),
  ('SMALL', 'SMALL', 'SMALL', 'MAX', 'BITOFF'),
  ('SMALL', 'SMALL', 'SMALL', 'UNIFORM', 'BITOFF'),
  ('SMALL', 'MAX', 'SMALL', 'UNIFORM', 'BITOFF'),
  ('SMALL', 'SMALL', 'SMALL', 'MIXED', 'MIN'),
  ('SMALL', 'SMALL', 'SMALL', 'LARGE', 'MAX'),
  ('MEDIUM', 'MIN', 'ANY', 'LARGE', 'BITOFF'),
  ('MEDIUM', 'HUGE', 'ANY', 'MIXED', 'BITOFF'),
  ('MEDIUM', 'MIN', 'ANY', 'ZERO', 'MAXVAL'),
  ('LARGE', 'MIN', 'ANY', 'LARGE', 'MAXVAL'),
  ('LARGE', 'MIN', 'ANY', 'LARGE', 'BITOFF'),
  ('LARGE', 'SMALL', 'MIN', 'MIXED', 'WIDE'),
  ('LARGE', 'SMALL', 'ANY', 'MIXED', 'MIN'),
  ('LARGE', 'HUGE', 'HUGE', 'UNIFORM', 'BITOFF'),
  ('LARGE', 'SMALL', 'ANY', 'UNIFORM', 'MIN'),
  ('LARGE', 'SMALL', 'ANY', 'MAX', 'MIN'),
  ('LARGE', 'SMALL', 'ANY', 'MIXED', 'MAX'),
  ('LARGE', 'SMALL', 'ANY', 'ONECOL1e-3', 'MAX'),
  ('LARGE', 'HUGE', 'ANY', 'ONECOL1', 'MAX'),
  ('LARGE', 'HUGE', 'ANY', 'ONEROW1', 'MAX'),
  ('LARGE', 'SMALL', 'ANY', 'ONECOL50', 'MAX'),
  ('LARGE', 'SMALL', 'ANY', 'ONEROW50', 'MAX'),
  ('HUGE', 'MIN', 'ANY', 'LARGE', 'MAXVAL'),
  ('HUGE', 'MIN', 'ANY', 'MIXED', 'BITOFF'),
  ('HUGE', 'HUGE', 'HUGE', 'UNIFORM', 'BITOFF'),
  ('HUGE', 'HUGE', 'HUGE', 'UNIFORM', 'WIDE'),
  ('HUGE', 'HUGE', 'HUGE', 'UNIFORM', 'MAX'),
  ('MAX', 'MAX', 'MAX', 'UNIFORM', 'MAXVAL'),
  ('MAX', 'MAX', 'MAX', 'UNIFORM', 'WIDE'),
  ('MAX', 'MAX', 'MAX', 'UNIFORM', 'MAX'),
  ('MAX', 'MAX', 'MAX', 'LARGE', 'MIN'),
  ('MAX', 'MAX', 'MAX', 'MIXED', 'WIDE'),
  ('MAX', 'MAX', 'MAX', 'ONECOL1e-3', 'MAX'),
  ('MAX', 'MAX', 'MAX', 'ONECOL1e-3', 'MIN'),
  ('MAX', 'MAX', 'MAX', 'ONECOL1', 'MAX'),
  ('MAX', 'MAX', 'MAX', 'ONECOL50', 'MAX'),
  ('MAX', 'MAX', 'MAX', 'ONEROW1e-3', 'MAX'),
  ('MAX', 'MAX', 'MAX', 'MAX', 'MAX'),
]

shell = []
sol_lines = {}
for sol in SOL_LOCATION:
  shell.append("g++ -Wall -O2 {} -o {}".format(full_path(sol), sol[0]))
  sol_lines[sol[0]] = []
shell.append("g++ -Wall -O2 ../input_format_validators/validate.cpp -o valid")

test_only = len(sys.argv) > 1 and sys.argv[1] == "TEST_ONLY"

def sol_time(sol, t):
  lam = solutions[sol]
  return lam(QRANGES[t[1]][1],
             NRANGES[t[0]][1] * NRANGES[t[0]][1],
             SRANGES[t[2]][1])

for (i, t) in enumerate(tests):
  tname = testName(i + 3, t[0], t[1], t[2], t[3], t[4])
  print "Generating {}".format(tname)
  if not test_only:
    printTest(tname, t[0], t[1], t[2], t[3], t[4])
  shell.append('echo "*************** {} *****************"'.format(tname))
  shell.append('./valid.exe < ./secret/{}.in'.format(tname))
  sols = [s[0] for s in SOL_LOCATION]
  sample_sol = 0
  while sol_time(sols[sample_sol], t) > 5e10:
    sample_sol += 1
  genSample, sample = output_sample(sols[sample_sol], tname)
  shell.extend(genSample)
  for s in sols:
    sol_lines[s].extend(output_line(s, tname, sample))

for sol in SOL_LOCATION:
  shell.append("echo \"***********************************\"")
  shell.append("echo \"Running solution {}\"".format(sol[0]))
  shell.extend(sol_lines[sol[0]])

for sol in SOL_LOCATION:
  shell.append("rm ./{}.exe".format(sol[0]))

f = open("./test.sh", "w")
for l in shell:
  f.write(l + "\n")

