/* Note - I'm running this on Windows, and that confuses strict mode as to the
 * correct shape of newlines. Since we use Linux newlines, I'm overriding the
 * target compilation mode to be linux.
 */
#define FOR_LINUX 1
#include "testlib.h"
#include <vector>
#include <set>
#include <utility>
#include <cstdio>
#include <cmath>

using namespace std;

typedef long double ld;

char nameBuf[100];

ld DP[500 * 500];

const ld cutoff = 1e-20;

/*
 * Note about errors - we are doing the cutoff trick (that is, we're
 * effectively rounding any probability smaller than 1e-20 down to zero). This
 * only introduces a downwards error (that is, the probabilities we compute
 * this way are lower than in reality), so if we compute that DP[T] is larger
 * than 1e-9, then it really is.
 * The non-cumulative total error (that is, the sum of all the undercounts)
 * is at most 2e-20. Since the values in the next step (aside from the new
 * non-cumulative error) are calcualted with positive coefficients from the
 * values in the previous step, and the coefficients for each old value sum up
 * to 1, the total error of the new step (before cutting off) is equal to the
 * total error of the old step. Thus, the total error introduced by cutting
 * off is N * M * 2e-20, which is 5e-16.
 */
void fillDP(const vector<double> &Ps, const vector<double> &Qs, int T) {
  for (int i = 1; i <= T; ++i) DP[i] = 0;
  DP[0] = 1;
  int from = 0;
  int to = 0;
  for (double p : Ps) {
    for (double q: Qs) {
      ld prob = p + q;
      for (int i = to + 1; i > from; i--) {
        DP[i] = prob * DP[i - 1] + (1. - prob) * DP[i];
      }
      DP[from] *= (1. - prob);
      if (DP[from] < cutoff) from++;
      if (DP[to+1] >= cutoff) to++;
    }
  }
}

int main(int argc, char **argv) {
  registerValidation(argc, argv);
  // readInt(minVal, maxVal, name);
  int M = inf.readInt(1, 500, "M");
  inf.readSpace();
  int N = inf.readInt(1, 500, "N");
  inf.readSpace();
  int T = inf.readInt(0, M * N, "T");
  inf.readSpace();
  int Q = inf.readInt(0, 500, "Q");
  inf.readEoln();
  // readStrictReals(size, minVal, maxVal, minDigitsAfterPoint, maxDigits, name)
  vector<double> Ps = inf.readStrictReals(M, 0, 0.1, 0, 6, "Ps");
  inf.readEoln();
  vector<double> Qs = inf.readStrictReals(N, 0, 0.1, 0, 6, "Qs");
  inf.readEoln();
  int minq = 1000;
  int maxq = -1;
  for (int i = 0; i < Q; ++i) {
    set<pair<int, int>> mines;
    sprintf(nameBuf, "S[%d]", i);
    int S = inf.readInt(0, 500, nameBuf);
    if (S > maxq) maxq = S;
    if (S < minq) minq = S;
    for (int j = 0; j < S; ++j) {
      inf.readSpace();
      sprintf(nameBuf, "First coordinate, %d'th mine, query %d", j, i);
      int x = inf.readInt(1, M, nameBuf);
      inf.readSpace();
      sprintf(nameBuf, "Second coordinate, %d'th mine, query %d", j, i);
      int y = inf.readInt(1, N, nameBuf);
      auto mine = make_pair(x, y);
      ensuref(mines.find(mine) == mines.end(),
              "Mine %d (%d, %d) in query %d is unique", j, x, y, i);
      mines.insert(mine);
    }
    inf.readEoln();
  }
  inf.readEof();
  // Verify the "the probability of getting at T mines is at least 1e-9"
  fillDP(Ps, Qs, T);
  ld expected = 0.;
  ld variance = 0.;
  for (double p: Ps) {
    for (double q: Qs) {
      expected += (p + q);
      variance += (p + q) - (p + q) * (p + q);
    }
  }
  fprintf(stderr,
          "%d x %d (= %d) field with %d mines (E = %.2Lf, StDev = %.2Lf)\n",
          M, N, M * N, T, expected, sqrtl(variance));
  fprintf(stderr,
          "%d queries, with %d-%d mines. Probability of %d mines: %.9Lf\n",
          Q, minq, maxq, T, DP[T]);
  inf.ensuref(DP[T] >= 1.000001e-5,
              "Value of DP[%d] should be > 1e-9, is %.20Lf", T, DP[T]);
  return 42;
}
