/* Naive solution to the Sweep Stakes problem.
 * Consists of a trivial DP to calculate the probability of getting X mines
 * in a specified set of K squares.
 * The formula we're looking for is the bayesian P(s = S | t = T) = 
 * P(s = S && t = T) // P(t == T) = P(s = S && (t-s) = T-S) / P(t = T)
 * This, in turn, from independence, is equal to
 * P(s = S) P(t-s = T-S) / P(t = T). */

// For all complexity calculations, Z = M * N. This algorithm is
// O(Q Z^2).

#include <vector>
#include <cstdio>
#include <utility>
#include <set>
#include <functional>
using namespace std;

typedef long double ld;

#define MAXN 500

int M, N, T, Qs;
ld P[MAXN];
ld Q[MAXN];

ld dataT[MAXN * MAXN];
ld dataS[MAXN];
ld dataTS[MAXN * MAXN];

// Fill_data is O(|vs|^2)
void fill_data(vector<pair<int, int>> vs, ld *data) {
  data[0] = 1.;
  int C = 0;
  for (auto v : vs) {
    int m = v.first;
    int n = v.second;
    ld prob = P[m] + Q[n];
    data[C + 1] = 0;
    for (int v = C + 1; v > 0; v--) {
      data[v] = (1. - prob) * data[v] + prob * data[v-1];
    }
    data[0] *= (1. - prob);
    C += 1;
  }
}

int main() {
  scanf("%d %d %d %d", &M, &N, &T, &Qs);
  for (int m = 0; m < M; m++) scanf("%Lf", &P[m]);
  for (int n = 0; n < N; n++) scanf("%Lf", &Q[n]);
  vector<pair<int, int>> all;
  for (int m = 0; m < M; ++m) for (int n = 0; n < N; ++n) {
    all.push_back(make_pair(m, n));
  }
  fill_data(all, dataT);
  vector<pair<int, int>> sel;
  set<pair<int, int>> cset;
  for (int q = 0; q < Qs; q++) {
    all.clear();
    sel.clear();
    int S;
    cset.clear();
    scanf("%d", &S);
    for (int s = 0; s < S; ++s) {
      int n, m;
      scanf("%d %d", &m, &n);
      cset.insert(make_pair(m - 1, n - 1));
    }
    for (int m = 0; m < M; m++) for (int n = 0; n < N; n++) {
      auto p = make_pair(m, n);
      if (cset.find(p) == cset.end()) {
        all.push_back(p);
      } else {
        sel.push_back(p);
      }
    }
    fill_data(sel, dataS);
    // This is the dominant factor - for each query, we run filling the 
    // T-S array in O(Z^2).
    fill_data(all, dataTS);
    for (int s = 0; s <= S; ++s) {
      if (s <= T) {
        printf("%.9Lf ", dataS[s] * dataTS[T - s] / dataT[T]);
      } else {
        printf("0 ");
      }
    }
    printf("\n");
  }
  return 0;
}
