/* Tree-based calculation solution to the Sweep Stakes problem.
 *
 * We calculate the probability of getting X mines in a specified set of K
 * squared by doing a tree-shaped calculation, where we calculate all the
 * probabilities of getting any number of mines from the sets of size 1, 2, 4,
 * etc.
 * The formula we're looking for is the bayesian P(s = S | t = T) = 
 * P(s = S && t = T) // P(t == T) = P(s = S && (t-s) = T-S) / P(t = T)
 * This, in turn, from independence, is equal to
 * P(s = S) P(t-s = T-S) / P(t = T). */

// For all complexity calculations, Z = M * N. This algorithm is
// O(Q Z^2).

#include <vector>
#include <cstdio>
#include <utility>
#include <set>
#include <functional>
using namespace std;

typedef long double ld;

#define MAXN 500

int M, N, T, Qs;
ld P[MAXN];
ld Q[MAXN];

ld dataT[2 * MAXN * MAXN];
ld dataS[2 * MAXN];
ld dataTS[2 * MAXN * MAXN];
ld temp[2][4 * MAXN * MAXN];

// Calculates the convolution of two arrays, i1 and i2, both of size X,
// into the array o (which has to have 2X space).
// o[i] = sum_j i1[j] * i2[i - j].
// The calculation is done by naive application of the formula.
void conv(int X, ld *i1, ld *i2, ld *o) {
  for (int i = 0; i < 2 * X; ++i) o[i] = 0;
  for (int i = 0; i < X; ++i) {
    for (int j = 0; j <= i; ++j) {
      o[i] += i1[j] * i2[i-j];
    }
  }
  for (int i = X; i < 2 * X; ++i) {
    for (int j = i + 1 - X; j < X; ++j) {
      o[i] += i1[j] * i2[i-j];
    }
  }
}

// Fill_data is O(|vs|^2)
void fill_data(vector<pair<int, int>> vs, ld *data) {
  // Instead of calculating data by adding one cell at a time, we'll be adding
  // them in a binary fashion. At the k-th step, we will calculate for each
  // i divisible by 2^k the probabilities of a s bombs being added in cells
  // i ... i + 2^k - 1.
  // We'll be subsequently merging these through standard convolution.
  int C = vs.size();
  for (int i = 0; i < C; ++i) {
    temp[0][2 * i] = (1 - P[vs[i].first] - Q[vs[i].second]);
    temp[0][2 * i + 1] = P[vs[i].first] + Q[vs[i].second];
  }
  int mskip = 1; while ((1 << (mskip - 1)) < C) mskip++;
  for (int i = C; i < (1 << (mskip - 1)); i++) {
    temp[0][2 * i] = 1.;
    temp[0][2 * i + 1] = 0.;
  }

  for (int step = 1, skip = 2; step < mskip; step += 1, skip *= 2) {
    int ol = (step - 1) & 1;
    int ne = step & 1;
    // The convolution for a particular value of beg is O(skip^2).
    // For a particular value of skip, we run this O(2^mskip / skip) times,
    // so the total complexity is (2^mskip * skip).
    // 2^mskip is O(C) (up to 2C).
    // Summing up over skip = 1,2,4,... we get the sum equal to
    // 2^mskip + (1 + 2 + 4 + ... + 2^mskip) = O(C^2).
    // This means we didn't change the overall complexity, but we did make the
    // algorithm more complex than the totally naive version. Yay!
    for (int beg = 0; beg < (1 << mskip); beg += 2 * skip) {
      conv(skip, &temp[ol][beg], &temp[ol][beg + skip], &temp[ne][beg]);
    }
  }

  for (int i = 0; i <= C; ++i) {
    data[i] = temp[(mskip + 1) & 1][i];
  }
}

int main() {
  scanf("%d %d %d %d", &M, &N, &T, &Qs);
  for (int m = 0; m < M; m++) scanf("%Lf", &P[m]);
  for (int n = 0; n < N; n++) scanf("%Lf", &Q[n]);
  vector<pair<int, int>> all;
  for (int m = 0; m < M; ++m) for (int n = 0; n < N; ++n) {
    all.push_back(make_pair(m, n));
  }
  fill_data(all, dataT);
  vector<pair<int, int>> sel;
  set<pair<int, int>> cset;
  for (int q = 0; q < Qs; q++) {
    all.clear();
    sel.clear();
    int S;
    cset.clear();
    scanf("%d", &S);
    for (int s = 0; s < S; ++s) {
      int n, m;
      scanf("%d %d", &m, &n);
      cset.insert(make_pair(m - 1, n - 1));
    }
    for (int m = 0; m < M; m++) for (int n = 0; n < N; n++) {
      auto p = make_pair(m, n);
      if (cset.find(p) == cset.end()) {
        all.push_back(p);
      } else {
        sel.push_back(p);
      }
    }
    fill_data(sel, dataS);
    // This is the dominant factor - for each query, we run filling the 
    // T-S array in O(Z^2).
    fill_data(all, dataTS);
    for (int s = 0; s <= S; ++s) {
      if (s <= T) {
        printf("%.9Lf ", dataS[s] * dataTS[T - s] / dataT[T]);
      } else {
        printf("0 ");
      }
    }
    printf("\n");
  }
  return 0;
}
