#include <cassert>
#include <cmath>
#include <complex>
#include <cstdio>
#include <iostream>
#include <vector>
using namespace std;

typedef long double real_t;

// Width of FFT (containing the vast majority of the probability distribution)
#define W 4096

// Fourier Transforms v[b..e-1] (requiring e-b = 2^n) over the field T using the given 2^n-th root of unity.
template<typename T> void FastFourierTransform(vector<T>& v, int b, int e, const T& root) {
  if (e-b <= 1) return;
  assert((e-b)%2 == 0);
  int m = (b+e)/2;
  {
    vector<T> tmp(m-b);
    for (int i = 0; i < m-b; i++) tmp[i] = v[b+2*i+1];
    for (int i = 1; i < m-b; i++) v[b+i] = v[b+2*i];
    for (int i = 0; i < m-b; i++) v[m+i] = tmp[i];
  }
  T root2 = root*root;
  FastFourierTransform(v, b, m, root2);
  FastFourierTransform(v, m, e, root2);
  T cur(1);
  for (int i = 0; i < m-b; i++) {
    if (i) cur *= root;
    T t = v[m+i] * cur;
    v[m+i] = v[b+i] - t;
    v[b+i] += t;
  }
}

void addMine(int n, real_t p, vector<real_t>& dp) {
  for (int i = n; i > 0; i--) dp[i] = (1-p)*dp[i] + p*dp[i-1];
  dp[0] *= 1-p;
}

int main() {
  int M, N, T, Q, S;
  complex<real_t> rt(cos(2*M_PI/W), sin(2*M_PI/W));
  while (cin >> M >> N >> T >> Q) {
    vector<real_t> Pi(M+1), Pj(N+1);
    for (int i = 1; i <= M; i++) cin >> Pi[i];
    for (int j = 1; j <= N; j++) cin >> Pj[j];

    vector<complex<real_t>> distc(W);
    distc[0] = 1.0;
    FastFourierTransform(distc, 0, W, rt);
    for (int i = 1; i <= M; i++) {
      vector<real_t> dp(N+1);
      dp[0] = 1.0;
      for (int j = 1; j <= N; j++) addMine(j, Pi[i]+Pj[j], dp);
      vector<complex<real_t>> dista(W);
      for (int i = 0; i <= N; i++) dista[i] = dp[i];
      FastFourierTransform(dista, 0, W, rt);
      for (int i = 0; i < W; i++) distc[i] *= dista[i];
    }

    while (Q--) {
      cin >> S;
      vector<real_t> dp(S+1);
      dp[0] = 1.0;
      for (int s = 1, i, j; s <= S; s++) {
        cin >> i >> j;
        addMine(s, Pi[i]+Pj[j], dp);
      }

      vector<complex<real_t>> dista(W), distb(W);
      for (int i = 0; i <= S; i++) dista[i] = dp[i];
      FastFourierTransform(dista, 0, W, rt);
      for (int i = 0; i < W; i++) if (dista[i] != (real_t) 0.0) distb[i] = distc[i] / dista[i];

      FastFourierTransform(distb, 0, W, ((real_t) 1.0)/rt);
      real_t tot = 0.0;
      for (int m = 0; m <= S; m++) tot += dp[m] * distb[(T+W-m)%W].real();
      for (int m = 0; m <= S; m++) {
        if (m) putchar(' ');
        printf("%.9Lf", dp[m] * distb[(T+W-m)%W].real() / tot);
      }
      putchar('\n');
    }
  }
}
