/* Tree-based, Fourier calculation solution to the Sweep Stakes problem.
 *
 * We calculate the probability of getting X mines in a specified set of K
 * squared by doing a tree-shaped calculation, where we calculate all the
 * probabilities of getting any number of mines from the sets of size 1, 2, 4,
 * etc. We calculate the convolution of two arrays by FFT.
 * The formula we're looking for is the bayesian P(s = S | t = T) = 
 * P(s = S && t = T) // P(t == T) = P(s = S && (t-s) = T-S) / P(t = T)
 * This, in turn, from independence, is equal to
 * P(s = S) P(t-s = T-S) / P(t = T).
 *
 * We will calculate P(T-S) from reverse Fourier as in reverse_fourier.cpp, but
 * we'll also take advantage of the CLT. P(T) is a sum of Z i.i.ds. The variance
 * of a single one of them is at most 0.25 (the maximum possible variance of a
 * 0-1 random variable - in practice it's actually at most 0.16, because p is
 * bounded by 0.2, but whatever).
 * So, the variance of the sum is 0.25 Z, and standard deviation is 0.5 sqrt(Z).
 * Now, CLT means roughly that the density of the sum decays roughly like
 * exp(-x^2 / 2), when normalized by the standard deviation. This means that the
 * values of dataT more than 5 sqrt(Z) away from the mean will be smaller than
 * 10^(-20).
 *
 * (Note - this is a horrifying misuse of the CLT, since Berry-Esseen bounds are
 * much, much weaker than that, they imply an error of roughly O(sqrt(Z)) on the
 * CLT approximation. However, the Berry-Esseen error occurs near the mean,
 * I assume that in this particular case I can derive a tail bound that behaves
 * roughly the way we want here)
 *
 * So, instead of applying the formula
 * P(T-S) = RF(F(P(T)) / F(P(S)))
 * to the actual P(T), we'll instead apply it to P(T) truncated to an area
 * where P(T) is actually above, say, 10^{-20}. This, as above, will be an
 * area of O(sqrt(Z)) size, and we'll apply the FFTs to that.
 */

// For all complexity calculations, Z = M * N. This algorithm is
// O(Q sqrt(Z) log Z + Z log^2 Z + Q S log^2 S).

#include <vector>
#include <cstdio>
#include <cmath>
#include <utility>
#include <set>
#include <functional>
#include <complex>
using namespace std;

typedef long double ld;
typedef complex<ld> cld;

#define MAXN 500
#define MAXS 500
#define TWOPI (4. * acosl(0.))

int M, N, T, Qs;
ld P[MAXN];
ld Q[MAXN];

ld dataT[2 * MAXN * MAXN];
ld dataS[2 * MAXN * MAXN];
ld dataTS[2 * MAXN * MAXN];
ld temp[2][4 * MAXN * MAXN];
cld tmp[4 * MAXN * MAXN];  // This is private for the DFT.
cld tmp2[4 * MAXN * MAXN];  // This is private for the conv.
cld td[4 * MAXN * MAXN];
cld sd[4 * MAXN * MAXN];

// Calculates the discrete Fourier transform of the array inp, which is
// supposed to have X. If rev is set to 1, calculates the inverse instead.
// Does this in place. This is more-or-less Cooley-Tukey.
void DFT(cld *inp, int X, int rev) {
  if (X == 1) return;
  for (int i = 0; i < X; ++i) tmp[i] = inp[i];
  for (int i = 0; i < X; ++i) 
    if (i & 1) inp[i/2 + X/2] = tmp[i];
    else inp[i/2] = tmp[i];
  cld *y0 = inp, *y1 = inp + X/2;
  DFT(y0, X/2, rev);
  DFT(y1, X/2, rev);
  ld alpha = TWOPI / (ld) X;
  for (int k = 0; k < X/2; k++) {
    cld mult = cld(cosl(k * alpha), (rev?-1:1) * sinl(k * alpha)) * y1[k];
    tmp[k] = y0[k] + mult;
    tmp[k + X/2] = y0[k] - mult;
  }
  for (int i = 0; i < X; ++i) inp[i] = tmp[i];
}

// Calculates the convolution of two arrays, i1 and i2, both of size X,
// into the array o (which has to have 2X space).
// o[i] = sum_j i1[j] * i2[i - j].
// The calculation is done by FFT, and so takes O(X log X).
void conv(int X, ld *i1, ld *i2, ld *o) {
  cld *ii1 = tmp2;
  cld *ii2 = tmp2 + 2*X;
  for (int i = 0; i < X; ++i) {
    ii1[i] = cld(i1[i], 0);
    ii2[i] = cld(i2[i], 0);
  }
  for (int i = X; i < 2*X; ++i) {
    ii1[i] = ii2[i] = cld(0, 0);
  }
  DFT(ii1, 2*X, 0);
  DFT(ii2, 2*X, 0);
  for (int i = 0; i < 2*X; ++i) {
    ii1[i] *= ii2[i];
  }
  DFT(ii1, 2*X, 1);
  for (int i = 0; i < 2 * X; ++i) {
    o[i] = ii1[i].real() / (2. * X);
  }
}

// Fill_data is O(|vs| log^2 |vs|)
void fill_data(vector<pair<int, int>> vs, ld *data) {
  // Instead of calculating data by adding one cell at a time, we'll be adding
  // them in a binary fashion. At the k-th step, we will calculate for each
  // i divisible by 2^k the probabilities of a s bombs being added in cells
  // i ... i + 2^k - 1.
  // We'll be subsequently merging these through FFT convolution.
  int C = vs.size();
  for (int i = 0; i < C; ++i) {
    temp[0][2 * i] = (1 - P[vs[i].first] - Q[vs[i].second]);
    temp[0][2 * i + 1] = P[vs[i].first] + Q[vs[i].second];
  }
  int mskip = 1; while ((1 << (mskip - 1)) < C) mskip++;
  for (int i = C; i < (1 << (mskip - 1)); i++) {
    temp[0][2 * i] = 1.;
    temp[0][2 * i + 1] = 0.;
  }

  for (int step = 1, skip = 2; step < mskip; step += 1, skip *= 2) {
    int ol = (step - 1) & 1;
    int ne = step & 1;
    // The convolution for a particular value of beg is O(skip log skip).
    // For a particular value of skip, we run this O(2^mskip / skip) times,
    // so the total complexity is (2^mskip * log skip).
    // 2^mskip is O(C) (up to 2C).
    // Summing up over skip = 1,2,4,... we get the sum equal to
    // 2^mskip + (1 + 2 + 3 + ... + mskip) = O(C log^2 C).
    // This is an actual improvement in complexity :)
    for (int beg = 0; beg < (1 << mskip); beg += 2 * skip) {
      conv(skip, &temp[ol][beg], &temp[ol][beg + skip], &temp[ne][beg]);
    }
  }

  for (int i = 0; i <= C; ++i) {
    data[i] = temp[(mskip + 1) & 1][i];
  }
}

int main() {
  scanf("%d %d %d %d", &M, &N, &T, &Qs);
  for (int m = 0; m < M; m++) scanf("%Lf", &P[m]);
  for (int n = 0; n < N; n++) scanf("%Lf", &Q[n]);
  vector<pair<int, int>> all;
  for (int m = 0; m < M; ++m) for (int n = 0; n < N; ++n) {
    all.push_back(make_pair(m, n));
  }
  fill_data(all, dataT);
  // Let's find the range where dataT > 1e-20.
  const ld CUTOFF = 1e-20;
  int from = -1;
  int to = -1;
  for (int i = 0; i <= M * N; ++i) {
    if (dataT[i] > CUTOFF) {
      to = i;
      if (from == -1) from = i;
    }
  }
  // fprintf(stderr, "The whole range is 0 to %d, the cutoff range from %d to %d\n", M * N, from, to);
  // "td" will hold the Fourier transform of dataT in this range.
  int TSize = 1; while (TSize < 2 * (to - from + 1) || TSize < 2 * MAXS) {
    TSize *= 2;
  }
  // fprintf(stderr, "Running the data on %d entries \n", TSize);
  const int mid = (from + to) / 2;
  for (int i = 0; i < TSize; ++i) {
    int d = mid + i - (TSize / 4);
    if (d < 0 || d >= M * N) {
      td[i] = cld(0, 0);
    } else {
      td[i] = cld(dataT[d], 0);
    }
  }
  DFT(td, TSize, 0);
  // fprintf(stderr, "The absolute value of zeroeth entry: (%.9Lf, %.9Lf)\n", td[0].real(), td[0].imag());

  vector<pair<int, int>> sel;
  for (int q = 0; q < Qs; q++) {
    sel.clear();
    int S;
    scanf("%d", &S);
    for (int s = 0; s < S; ++s) {
      int n, m;
      scanf("%d %d", &m, &n);
      sel.push_back(make_pair(m - 1, n - 1));
    }
    fill_data(sel, dataS);
    
    // Do the reverse FFT trick on the bounded range.
    for (int i = 0; i <= S; ++i) sd[i] = cld(dataS[i], 0);
    for (int i = S+1; i < TSize; ++i) sd[i] = cld(0, 0);
    DFT(sd, TSize, 0);
    for (int i = 0; i < TSize; ++i) {
      sd[i] = (abs(sd[i]) > 1e-10) ? td[i] / sd[i] : 0;
    }
    DFT(sd, TSize, 1);
    for (int i = from; i <= to; ++i) {
      int d = i - mid + (TSize / 4);
      if (d < 0 || d >= TSize) {
        dataTS[i] = 0;
      } else {
        dataTS[i] = sd[d].real() / TSize;
      }
    }
    for (int s = 0; s <= S; ++s) {
      if (s <= T && T - s >= from && T - s <= to) {
        printf("%.9Lf ", dataS[s] * dataTS[T - s] / dataT[T]);
      } else {
        printf("0 ");
      }
    }
    printf("\n");
  }
  return 0;
}
