/* Tree-based, Fourier calculation solution to the Sweep Stakes problem.
 *
 * We calculate the probability of getting X mines in a specified set of K
 * squared by doing a tree-shaped calculation, where we calculate all the
 * probabilities of getting any number of mines from the sets of size 1, 2, 4,
 * etc. We calculate the convolution of two arrays by FFT.
 * The formula we're looking for is the bayesian P(s = S | t = T) = 
 * P(s = S && t = T) // P(t == T) = P(s = S && (t-s) = T-S) / P(t = T)
 * This, in turn, from independence, is equal to
 * P(s = S) P(t-s = T-S) / P(t = T). */

// For all complexity calculations, Z = M * N. This algorithm is
// O(Q Z log^2 Z).

#include <vector>
#include <cstdio>
#include <cmath>
#include <utility>
#include <set>
#include <functional>
#include <complex>
using namespace std;

typedef long double ld;
typedef complex<ld> cld;

#define MAXN 500
#define TWOPI (4. * acosl(0.))

int M, N, T, Qs;
ld P[MAXN];
ld Q[MAXN];

ld dataT[2 * MAXN * MAXN];
ld dataS[2 * MAXN];
ld dataTS[2 * MAXN * MAXN];
ld temp[2][4 * MAXN * MAXN];
cld tmp[4 * MAXN * MAXN];  // This is private for the DFT.
cld tmp2[4 * MAXN * MAXN];  // This is private for the conv.

// Calculates the discrete Fourier transform of the array inp, which is
// supposed to have X. If rev is set to 1, calculates the inverse instead.
// Does this in place. This is more-or-less Cooley-Tukey.
void DFT(cld *inp, int X, int rev) {
  if (X == 1) return;
  for (int i = 0; i < X; ++i) tmp[i] = inp[i];
  for (int i = 0; i < X; ++i) 
    if (i & 1) inp[i/2 + X/2] = tmp[i];
    else inp[i/2] = tmp[i];
  cld *y0 = inp, *y1 = inp + X/2;
  DFT(y0, X/2, rev);
  DFT(y1, X/2, rev);
  ld alpha = TWOPI / (ld) X;
  for (int k = 0; k < X/2; k++) {
    cld mult = cld(cosl(k * alpha), (rev?-1:1) * sinl(k * alpha)) * y1[k];
    tmp[k] = y0[k] + mult;
    tmp[k + X/2] = y0[k] - mult;
  }
  for (int i = 0; i < X; ++i) inp[i] = tmp[i];
}

// Calculates the convolution of two arrays, i1 and i2, both of size X,
// into the array o (which has to have 2X space).
// o[i] = sum_j i1[j] * i2[i - j].
// The calculation is done by FFT, and so takes O(X log X).
void conv(int X, ld *i1, ld *i2, ld *o) {
  cld *ii1 = tmp2;
  cld *ii2 = tmp2 + 2*X;
  for (int i = 0; i < X; ++i) {
    ii1[i] = cld(i1[i], 0);
    ii2[i] = cld(i2[i], 0);
  }
  for (int i = X; i < 2*X; ++i) {
    ii1[i] = ii2[i] = cld(0, 0);
  }
  DFT(ii1, 2*X, 0);
  DFT(ii2, 2*X, 0);
  for (int i = 0; i < 2*X; ++i) {
    ii1[i] *= ii2[i];
  }
  DFT(ii1, 2*X, 1);
  for (int i = 0; i < 2 * X; ++i) {
    o[i] = ii1[i].real() / (2. * X);
  }
}

// Fill_data is O(|vs| log^2 |vs|)
void fill_data(vector<pair<int, int>> vs, ld *data) {
  // Instead of calculating data by adding one cell at a time, we'll be adding
  // them in a binary fashion. At the k-th step, we will calculate for each
  // i divisible by 2^k the probabilities of a s bombs being added in cells
  // i ... i + 2^k - 1.
  // We'll be subsequently merging these through FFT convolution.
  int C = vs.size();
  for (int i = 0; i < C; ++i) {
    temp[0][2 * i] = (1 - P[vs[i].first] - Q[vs[i].second]);
    temp[0][2 * i + 1] = P[vs[i].first] + Q[vs[i].second];
  }
  int mskip = 1; while ((1 << (mskip - 1)) < C) mskip++;
  for (int i = C; i < (1 << (mskip - 1)); i++) {
    temp[0][2 * i] = 1.;
    temp[0][2 * i + 1] = 0.;
  }

  for (int step = 1, skip = 2; step < mskip; step += 1, skip *= 2) {
    int ol = (step - 1) & 1;
    int ne = step & 1;
    // The convolution for a particular value of beg is O(skip log skip).
    // For a particular value of skip, we run this O(2^mskip / skip) times,
    // so the total complexity is (2^mskip * log skip).
    // 2^mskip is O(C) (up to 2C).
    // Summing up over skip = 1,2,4,... we get the sum equal to
    // 2^mskip + (1 + 2 + 3 + ... + mskip) = O(C log^2 C).
    // This is an actual improvement in complexity :)
    for (int beg = 0; beg < (1 << mskip); beg += 2 * skip) {
      conv(skip, &temp[ol][beg], &temp[ol][beg + skip], &temp[ne][beg]);
    }
  }

  for (int i = 0; i <= C; ++i) {
    data[i] = temp[(mskip + 1) & 1][i];
  }
}

int main() {
  scanf("%d %d %d %d", &M, &N, &T, &Qs);
  for (int m = 0; m < M; m++) scanf("%Lf", &P[m]);
  for (int n = 0; n < N; n++) scanf("%Lf", &Q[n]);
  vector<pair<int, int>> all;
  for (int m = 0; m < M; ++m) for (int n = 0; n < N; ++n) {
    all.push_back(make_pair(m, n));
  }
  fill_data(all, dataT);
  vector<pair<int, int>> sel;
  set<pair<int, int>> cset;
  for (int q = 0; q < Qs; q++) {
    all.clear();
    sel.clear();
    int S;
    cset.clear();
    scanf("%d", &S);
    for (int s = 0; s < S; ++s) {
      int n, m;
      scanf("%d %d", &m, &n);
      cset.insert(make_pair(m - 1, n - 1));
    }
    for (int m = 0; m < M; m++) for (int n = 0; n < N; n++) {
      auto p = make_pair(m, n);
      if (cset.find(p) == cset.end()) {
        all.push_back(p);
      } else {
        sel.push_back(p);
      }
    }
    fill_data(sel, dataS);
    // This is the dominant factor - for each query, we run filling the 
    // T-S array in O(Z log^2 Z).
    fill_data(all, dataTS);

    for (int s = 0; s <= S; ++s) {
      if (s <= T) {
        printf("%.9Lf ", dataS[s] * dataTS[T - s] / dataT[T]);
      } else {
        printf("0 ");
      }
    }
    printf("\n");
  }
  return 0;
}
