#include <bits/stdc++.h>
#include <CGAL/Interval_nt.h>

using namespace std;
using namespace CGAL;

using real_t = Interval_nt<>;
using complex_t = complex<real_t>;
static constexpr int FFT_SIZE = 4096;

static real_t ld_to_real(long double x)
{
    double y = x;
    if (y == x)
        return real_t(y);
    else if (y < x)
        return real_t(y, nextafter(y, INFINITY));
    else
        return real_t(nextafter(y, -INFINITY), y);
}

static real_t cosi(real_t angle)
{
    real_t lo = ld_to_real(cosl(angle.inf()));
    real_t hi = ld_to_real(cosl(angle.sup()));
    return real_t(std::min(lo.inf(), hi.inf()), std::max(lo.sup(), hi.sup()));
}

static real_t sini(real_t angle)
{
    real_t lo = ld_to_real(sinl(angle.inf()));
    real_t hi = ld_to_real(sinl(angle.sup()));
    return real_t(std::min(lo.inf(), hi.inf()), std::max(lo.sup(), hi.sup()));
}

static complex_t make_root(int idx, int len)
{
    real_t angle = ld_to_real(-2.0L * M_PIl * idx / len);
    return complex_t(cosi(angle), sini(angle));
}

template<typename Traits, typename SizeType = unsigned int>
class FFT
{
public:
    typedef SizeType size_type;
    typedef typename Traits::type type;

private:
    std::vector<std::vector<type>> roots, rroots;
    std::vector<size_type> butterfly;
    type inv_size;

    void permute(std::vector<type> &a) const
    {
        const size_type S = size();
        assert(a.size() == S);
        for (size_type i = 0; i < S; i++)
        {
            size_type other = butterfly[i];
            if (i < other)
                std::swap(a[i], a[other]);
        }
    }

    void transform(std::vector<type> &a, bool invert) const
    {
        const std::vector<std::vector<type>> &allr = invert ? rroots : roots;
        const size_type S = size();

        permute(a);
        for (std::size_t i = 1; i < allr.size(); i++)
        {
            const std::vector<type> &r = allr[i];
            size_type outer = size_type(1) << i;
            size_type inner = outer / 2;
            for (size_type j = 0; j < S; j += outer)
                for (size_type k = 0; k < inner; k++)
                {
                    type x = a[j + k];
                    type y = a[j + k + inner] * r[k];
                    a[j + k] = x + y;
                    a[j + k + inner] = x - y;
                }
        }
    }

    void init_roots(size_type len)
    {
        if (len == 1)
        {
            roots.emplace_back();
            roots.back().push_back(complex_t(1, 0));
            rroots.push_back(roots.back());
        }
        else
        {
            init_roots(len >> 1);
            std::vector<type> new_roots;
            new_roots.reserve(len);
            for (size_type i = 0; i < len; i++)
                new_roots.push_back(make_root(i, len));
            std::vector<type> new_rroots;
            new_rroots.reserve(len);
            new_rroots.push_back(new_roots[0]);
            new_rroots.insert(new_rroots.end(), new_roots.rbegin(), new_roots.rend() - 1);
            roots.push_back(std::move(new_roots));
            rroots.push_back(std::move(new_rroots));
        }
    }

    void init_butterfly(size_type len)
    {
        butterfly.resize(len);
        for (size_type i = 1; i < len; i <<= 1)
            for (size_type j = 0; j < i; j++)
            {
                butterfly[j] <<= 1;
                butterfly[i + j] = butterfly[j] + 1;
            }
    }

public:
    size_type size() const { return butterfly.size(); }

    FFT(size_type len)
    {
        assert(len > 0 && (len & (len - 1)) == 0);
        init_roots(len);
        init_butterfly(len);
        inv_size = Traits::inverse(complex_t(len, 0));
    }

    void fft(std::vector<type> &a) const
    {
        transform(a, false);
    }

    void rfft(std::vector<type> &a) const
    {
        transform(a, true);
        for (type &v : a)
            v *= inv_size;
    }
};

template<typename T>
class fft_traits_complex
{
public:
    typedef std::complex<T> type;

    static type inverse(type value)
    {
        return std::complex<T>(1, 0) / value;
    }
};

typedef FFT<fft_traits_complex<real_t>> FFTr;

static inline complex_t cmul(const complex_t &a, const complex_t &b)
{
    return complex_t(a.real() * b.real() - a.imag() * b.imag(),
                     a.real() * b.imag() + a.imag() * b.real());
}

static vector<complex_t> combine(
    const vector<real_t> &probs, const vector<complex_t> &prob_f,
    int A, int B)
{
    vector<complex_t> ans(FFT_SIZE);
    if (B - A == 1)
    {
        real_t p = probs[A];
        complex_t q(real_t(1) - p, 0);
        for (int k = 0; k < FFT_SIZE; k++)
            ans[k] = p * prob_f[k] + q;
    }
    else
    {
        int M = (A + B) / 2;
        auto left = combine(probs, prob_f, A, M);
        auto right = combine(probs, prob_f, M, B);
        for (int k = 0; k < FFT_SIZE; k++)
            ans[k] = cmul(left[k], right[k]);
    }
    return ans;
}

static real_t read_real()
{
    long double x;
    cin >> x;
    return ld_to_real(x);
}

static double rel_width(const complex_t &x)
{
    complex<double> s(median(x.real()), median(x.imag()));
    complex<double> bias(width(x.real()), width(x.imag()));
    return s != 0.0 ? abs(bias) / abs(s) : 0.0;
}

int main()
{
    ios::sync_with_stdio(false);
    int R, C, T, Q;
    cin >> R >> C >> T >> Q;
    vector<real_t> pr(R), qc(C);
    for (int i = 0; i < R; i++)
        pr[i] = read_real();
    for (int i = 0; i < C; i++)
        qc[i] = read_real();

    FFTr plan(FFT_SIZE);
    vector<complex_t> prob_f(FFT_SIZE);
    for (int i = 0; i < FFT_SIZE; i++)
        prob_f[i] = make_root(i, FFT_SIZE);

    vector<real_t> probs;
    probs.reserve(R * C);
    for (int i = 0; i < R; i++)
        for (int j = 0; j < C; j++)
            probs.push_back(pr[i] + qc[j]);
    vector<complex_t> full_f = combine(probs, prob_f, 0, probs.size());

    vector<complex_t> full = full_f;
    plan.rfft(full);
    real_t pT = full[T % FFT_SIZE].real();
    cerr << "p[T]: " << pT << '\n';
    real_t norm = real_t(1) / pT;

    cout << fixed << setprecision(12);
    double max_width = 0;
    for (int i = 0; i < Q; i++)
    {
        int S;
        cin >> S;
        probs.resize(S);
        for (int j = 0; j < S; j++)
        {
            int r, c;
            cin >> r >> c;
            r--;
            c--;
            probs[j] = pr[r] + qc[c];
        }
        vector<complex_t> inner = combine(probs, prob_f, 0, probs.size());
        vector<complex_t> outer(FFT_SIZE);
        for (int k = 0; k < FFT_SIZE; k++)
            outer[k] = full_f[k] / inner[k];
        plan.rfft(inner);
        plan.rfft(outer);
        for (int j = 0; j <= S; j++)
        {
            int r = T - j;
            real_t ans = 0;
            if (r >= 0 && r <= R * C - S)
                ans = outer[r % FFT_SIZE].real() * inner[j % FFT_SIZE].real() * norm;
            max_width = std::max(max_width, width(ans));
            cout << median(ans) << ' ';
        }
        cout << '\n';
    }
    cerr << setprecision(12) << "Max width: " << max_width << '\n';
}
