#!/usr/bin/env python3

import copy
import math
import random
from typing import Tuple, List, Optional, Iterator, Iterable, overload


class Vector:
    def __init__(self, data: List[float]) -> None:
        self._data = data

    @classmethod
    def zeros(cls, N: int) -> 'Vector':
        return Vector([0.0] * N)

    def __add__(self, other: 'Vector') -> 'Vector':
        return Vector([a + b for a, b in zip(self._data, other._data)])

    def __sub__(self, other: 'Vector') -> 'Vector':
        return Vector([a - b for a, b in zip(self._data, other._data)])

    def __mul__(self, other: float) -> 'Vector':
        return Vector([a * other for a in self._data])

    def __rmul__(self, other: float) -> 'Vector':
        return self * other

    def __neg__(self) -> 'Vector':
        return Vector([-a for a in self._data])

    def __pos__(self) -> 'Vector':
        return Vector(list(self._data))

    def __abs__(self) -> 'Vector':
        return Vector([abs(x) for x in self._data])

    @overload
    def __getitem__(self, idx: int) -> float: ...
    @overload
    def __getitem__(self, idx: slice) -> 'Vector': ...

    def __getitem__(self, idx):
        if isinstance(idx, slice):
            return Vector(self._data[idx])
        else:
            return self._data[idx]

    def __setitem__(self, idx: int, value: float) -> None:
        self._data[idx] = value

    def __len__(self) -> int:
        return len(self._data)

    def __iter__(self) -> Iterator[float]:
        return iter(self._data)

    def __bool__(self) -> bool:
        return all(self._data)

    def __str__(self) -> str:
        return '[' + '\n'.join('{:>10}'.format(repr(x)) for x in self._data) + ']'


def dot(a: Iterable[float], b: Iterable[float]) -> float:
    return sum((x * y for x, y in zip(a, b)), 0.0)


class Matrix:
    def __init__(self, data: List[List[float]]) -> None:
        assert len(data) > 0
        assert len(data[0]) > 0
        for row in data:
            assert len(row) == len(data[0])
        self._data = data

    @classmethod
    def zeros(cls, R: int, C: int) -> 'Matrix':
        assert R > 0 and C > 0
        return Matrix([[0.0] * C for _ in range(R)])

    @classmethod
    def eye(cls, R: int) -> 'Matrix':
        return Matrix([[(1.0 if i == j else 0.0) for j in range(R)] for i in range(R)])

    @property
    def shape(self) -> Tuple[int, int]:
        return (len(self._data), len(self._data[0]))

    @property
    def T(self) -> 'Matrix':
        R, C = self.shape
        return Matrix([[self[j, i] for j in range(R)] for i in range(C)])

    def __add__(self, other: 'Matrix') -> 'Matrix':
        assert self.shape == other.shape
        out = [[a + b for a, b in zip(A, B)] for A, B in zip(self._data, other._data)]
        return Matrix(out)

    def __sub__(self, other: 'Matrix') -> 'Matrix':
        assert self.shape == other.shape
        out = [[a - b for a, b in zip(A, B)] for A, B in zip(self._data, other._data)]
        return Matrix(out)

    def __neg__(self) -> 'Matrix':
        return Matrix([[-a for a in row] for row in self._data])

    def __pos__(self) -> 'Matrix':
        return Matrix(copy.deepcopy(self._data))

    @overload
    def __matmul__(self, other: 'Matrix') -> 'Matrix': ...
    @overload
    def __matmul__(self, other: 'Vector') -> 'Vector': ...

    def __matmul__(self, other):
        if isinstance(other, Matrix):
            assert self.shape[1] == other.shape[0]
            out = self.zeros(self.shape[0], other.shape[1])
            d = self.shape[1]
            for i in range(self.shape[0]):
                for j in range(other.shape[1]):
                    for k in range(d):
                        out._data[i][j] += self[i, k] * other[k, j]
            return out
        else:
            return Vector([dot(row, other) for row in self._data])

    def __rmatmul__(self, other: Vector) -> Vector:
        return self.T @ other

    def __mul__(self, other: float) -> 'Matrix':
        return Matrix([[a * other for a in row] for row in self._data])

    def __rmul(self, other: float) -> 'Matrix':
        return self * other

    def __getitem__(self, idx: Tuple[int, int]) -> float:
        return self._data[idx[0]][idx[1]]

    def __setitem__(self, idx: Tuple[int, int], value: float) -> None:
        self._data[idx[0]][idx[1]] = value

    def swap_rows(self, r1: int, r2: int) -> None:
        self._data[r1], self._data[r2] = self._data[r2], self._data[r1]

    def __str__(self) -> str:
        rows = ['[' + ' '.join('{:>10}'.format(repr(a)) for a in row) + ']' for row in self._data]
        return '\n'.join(rows)

    def __bool__(self) -> bool:
        return all(all(row) for row in self._data)


def gauss_jordan(A: Matrix, b: Vector) -> Tuple[List[Tuple[int, int]], List[int]]:
    """Destructively apply Gauss-Jordan algorithm.

    The returned matrix has leading -1's in each row.

    Returns
    -------
    fixed
        Matrix indices of the leading -1's.
    free
        Indices of free variables.
    """
    R, C = A.shape
    assert len(b) == R
    r = 0
    fixed = []
    free = []
    for c in range(C):
        p = r
        for i in range(p, R):
            if abs(A[i, c]) > abs(A[p, c]):
                p = i
        if p == R:
            free.append(c)
            continue
        A.swap_rows(p, r)
        b[p], b[r] = b[r], b[p]
        s = -1.0 / A[r, c]
        for j in range(c, C):
            A[r, j] *= s
        b[r] *= s
        for i in range(r + 1, R):
            s = A[i, c]
            for j in range(c, C):
                A[i, j] += s * A[r, j]
            b[i] += s * b[r]
        fixed.append((r, c))
        r += 1

    for r, c in fixed:
        for i in range(r):
            s = A[i, c]
            for j in range(c, C):
                A[i, j] += s * A[r, j]
            b[i] += s * b[r]

    return fixed, free


def solve(A: Matrix, b: Vector) -> Tuple[Vector, Optional[Matrix]]:
    """Destructively solve Ax = b.

    A solution must exist.

    Returns
    -------
    x0
        A single solution.
    N
        Matrix whose columns form a basis for the null space (not necessarily
        orthogonal), or None if the null-space has zero dimensions.
    """
    old_A = +A
    old_b = +b
    R, C = A.shape
    fixed, free = gauss_jordan(A, b)
    empty = len(free) - (C - R)     # Number of rows in A that are all zero
    assert empty == 0 or all(x == 0 for x in b[-empty:])
    x0 = Vector.zeros(C)
    for r, c in fixed:
        # We've already done back-propagation, and free variables are all zero
        x0[c] = -b[r]
    # assert not old_A @ x0 - old_b
    if not free:
        return x0, None
    N = Matrix.zeros(C, len(free))
    for i, f in enumerate(free):
        N[f, i] = 1.0
    for r, c in fixed:
        for i, f in enumerate(free):
            N[c, i] = A[r, f]
    # assert not old_A @ N
    return x0, N


def solve_vectors(A: Matrix, b: Vector) -> Vector:
    # old_A = +A
    # old_b = +b
    R, C = A.shape
    x0, N = solve(A, b)
    if N is not None:
        Qx = Matrix.eye(C)
        Qx[-1, -1] = 0.0
        rx = Vector.zeros(C)
        rx[-1] = -0.5
        dx = 0.0

        F = N.shape[1]
        rQ = Qx @ N
        Q = rQ.T @ rQ
        r = x0 @ rQ + rx @ N
        d = dot(x0[:-1], x0[:-1]) - x0[-1] + dx   # x0.T @ Q @ x0 + 2rx.T @ x0 + dx
        # z.T @ Q @ z + 2r.T @ z + d = 0
        s, _ = solve(+Q, -r)
        c = dot(Q @ s, s) + dot(r, s) * 2.0 + d
        y = Vector.zeros(F)
        idx = 0
        while not Q[idx, idx]:
            idx += 1
        y[idx] = math.sqrt(float(-c / Q[idx, idx]))
        z = y + s
        x = x0 + N @ z
        # assert not (old_A @ x - old_b)
        # print(float(abs(dot(x, Qx @ x) + 2 * dot(rx, x))))
        # assert abs(dot(x, Qx @ x) + 2 * dot(rx, x)) < 1e-9
        return x
    else:
        return x0


def main():
    d, n = [int(x) for x in input().split()]
    n = min(n, d + 1)
    A: List[List[float]] = []
    b = Vector.zeros(n)
    bias = [random.random() for _ in range(d)]
    bias.append(0.0)
    for i in range(n):
        row = [float(x) + b_ for x, b_ in zip(input().split(), bias)]
        u = row[:-1]
        d = row[-1]
        coeff = [-2.0 * ui for ui in u]
        coeff.append(1.0)
        A.append(coeff)
        b[i] = d * d - dot(u, u)
    ans = solve_vectors(Matrix(A), b)
    print(*(x - b_ for x, b_ in zip(ans[:-1], bias[:-1])))


if __name__ == '__main__':
    main()
