#!/usr/bin/env python3

import sys

# find a solution of a system of multivariate quadratic equations where
# - no term contains more than one variable
# - all quadratic terms have coefficient 1

eps = 1e-7

def dot_product(u, v):
  return sum(u[i] * v[i] for i in range(len(u)))

def quad(u):
  return dot_product(u, u)

def add(u, c, v):
  return [ u[i] + c * v[i] for i in range(len(u)) ]

def scalar_mult(c, u):
  return [ c * u[i] for i in range(len(u)) ]

def nproj(u, v):
  f = dot_product(u, v) / dot_product(u, u)
  return scalar_mult(f, u)

d, n = map(int, sys.stdin.readline().split())
inp = [ list(map(float, sys.stdin.readline().split())) for _ in range(n) ]

p = min(range(n), key=lambda i: inp[i][d])
inp[0], inp[p] = inp[p], inp[0]

trans = inp[0][:-1]
# mat = the linear parts of the equations; constants in the last column
mat = []
for row in inp:
  xs, e = row[:-1], row[-1]
  xs = add(xs, -1, trans) # translate points so first becomes origin
  c = quad(xs) - e * e
  mat.append(scalar_mult(-2, xs) + [c])

# subtracting first equation from the others eliminates their quadratic terms
# linear part of first equation is 0
for i in range(1, n):
  mat[i][d] -= mat[0][d]

# solve system of n-1 linear equations by Gauss-Jordan elimination
eq = [None] * d # equation of non-free variable
row, col = 1, 0
while row < n and col < d:
  p = max(range(row, n), key=lambda i: abs(mat[i][col]))
  if abs(mat[p][col]) > eps:
    mat[p], mat[row] = mat[row], mat[p]
    for i in range(row + 1, n):
      f = mat[i][col] / mat[row][col]
      for j in range(col, d + 1):
        mat[i][j] -= f * mat[row][j]
    eq[col] = row
    row += 1
  col += 1
if row < n:
  mat = mat[:row]

# backward substitution
for col in range(d - 1, 0, -1):
  row = eq[col]
  if row != None:
    for i in range(1, row):
      f = mat[i][col] / mat[row][col]
      for j in range(col, d + 1):
        mat[i][j] -= f * mat[row][j]

if None not in eq:
  # no free variables
  sol = [ -mat[i + 1][d] / mat[i + 1][i] for i in range(d) ]
else:
  # 1. compute vector representation of the affine subspace
  # 1.1 some vector in the space by setting all free variables to 0
  offset = [ 0 if eq[i] == None else -(mat[eq[i]][d] / mat[eq[i]][i]) for i in range(d) ]
  # 1.2 a vector basis by setting each free variable to 1, respectively, and subtracting the origin of the space
  basis = [ [ (1 if i == j else 0) if eq[i] == None else -mat[eq[i]][j] / mat[eq[i]][i] for i in range(d) ] for j in range(d) if eq[j] == None ]
  # 1.3 orthonormalise the basis using the Gram-Schmidt process
  for i in range(len(basis)):
    for j in range(i):
      basis[i] = add(basis[i], -1, nproj(basis[j], basis[i]))
  # 2. find closest point to origin by projecting origin to affine space
  # 2.1 translate from affine space to vector space
  trans_offset = scalar_mult(-1, offset)
  # 2.2 project translated origin to vector space
  closest = [0] * d
  for u in basis:
    closest = add(closest, 1, nproj(u, trans_offset))
  # 2.3 translate result back to affine space
  closest = add(closest, 1, offset)
  # 3. binary search for the correct distance along any line in the affine subspace through the closest point
  hi = 1
  while quad(add(closest, hi, basis[0])) <= -mat[0][d]:
    hi *= 2
  lo = 0
  while True:
    med = (hi + lo) / 2
    if not (lo < med < hi):
      break
    if quad(add(closest, med, basis[0])) <= -mat[0][d]:
      lo = med
    else:
      hi = med
  sol = add(closest, lo, basis[0])

# translate back
sol = add(sol, 1, trans)

for i in range(d):
  print('{:.17f}'.format(sol[i]))

def abs_or_rel(x, y):
  err = abs(x - y)
  if abs(y) > 1:
    err /= abs(y)
  if err < 1e-6:
    return True
  print(x, y, err)
  return False

for i in range(n):
  assert abs_or_rel(quad(add(inp[i][:-1], -1, sol)) ** 0.5, inp[i][-1])

